#ifndef BSP_H
#define BSP_H

#include "polygon.h"
#include "lightmap.h"
#include "portal.h"
#include "tll.h"
#include "decal.h"

// Enumerated constants for the bsp and portal functions
enum {Front, Back, TwoFrontOneBack, OneFrontTwoBack, OneFrontOneBack, PortalWasSplit, OnPartition};

typedef struct BSP_node
{
    int nodeid;             //unique identification number given to the node
    bool visible;           //flag used as part of the pvs
    POLYGON partition;      //we'll use this to store the partitioning plane that divides our nodes up
    BSP_node* backnode;     //pointer to the node that is on the back side of our partitioning plane
    BSP_node* frontnode;    //pointer to the node that is on the front side of our partitioning plane
    bool leaf;              //true or false value for a node saying that this node has no children and only contains polygons
    int numpolys;           //number of polygons in the node, this will be 0 if the node is not a leaf
    POLYGON* nodepolylist;  //pointer to the list of polygons in the leaf node, again this list will be empty if the node is not a leaf
    Lightmap* nodelightmaplist; //pointer to a list of lightmaps or NULL if this node isn't a leaf
    int numportals;             //the number of portals in the leaf node
    LinkedList<PORTAL> portallist; //list of portals in the leaf node
    int numdecals;
    LinkedList<DECAL> decallist; //list of decals in the leaf node
} BSP_node;

int FindCurrentLeaf(VECTOR Position, BSP_node* node);
int SplitPolygon(POLYGON, POLYGON, POLYGON*);
int SelectPartitionfromList(POLYGON* nodepolylist, int numpolys, int* bestfront, int* bestback);
void BuildBSP(BSP_node *node);
int RenderBSP(BSP_node *node);
void DeleteBSP(BSP_node *node);
void DrawIntersectionSphere(VECTOR coordinates);

#endif
