#include <windows.h>
#include "bullet.h"
#include "decal.h"
#include "vector.h"
#include "vertex.h"
#include "polygon.h"
#include "texture.h"
#include "listnode.h"
#include "collision.h"
#include "general.h"
#include "camera.h"
#include "bsp.h"
#include "particle.h"


extern VERTEX* vertex;
extern TEXTURE* texture;
extern int numBullets;
extern BULLET* bullet;
extern int numBulletsActive;
extern int numleaves;
extern LinkedList<ListNode> LeafList;
extern LinkedList<ListNode> PartitionList;
extern ListNode* listnode;
extern CAMERA* camera;
extern int currentCamera;
extern int bulletspeed;
extern int bullettype;
extern BSP_node* root;
extern int maxDecals;
extern DECAL muzzleflashdecal;
extern DECAL impactflashdecal;

BULLET::BULLET()
{
}

BULLET::~BULLET()
{
}

void BULLET::Reset()
{
    Orientation.Reset();
    Position.Reset();
        Delta_x = 0.0;
        Delta_y = 0.0;
        Delta_z = 0.0;
        Matrix.LoadIdentity();
}

void BULLET::Initialize()
{
    modelvertlist = new VERTEX[numverts];
    modelpolylist = new POLYGON[numpolys];
}

void BULLET::Draw()
{
    glPushMatrix();
    glTranslatef(Position.x,Position.y,Position.z);
    Matrix.QuatToMatrix(Orientation);
    glMultMatrixf(Matrix.Element);

    float mat_ambient[] = { 0.3, 0.3, 0.3, 1.0 };
    float mat_diffuse[] = { 0.3, 0.3, 0.3, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

    glDisable(GL_TEXTURE_2D);
    for (int loop = 0; loop < numpolys; loop++)
    {
        glBegin(GL_TRIANGLES);
            glNormal3fv(&modelpolylist[loop].Vertex[0].nx);
            glTexCoord2f(modelpolylist[loop].Vertex[0].u, modelpolylist[loop].Vertex[0].v);
            glVertex3fv(&modelpolylist[loop].Vertex[0].x);
            glTexCoord2f(modelpolylist[loop].Vertex[1].u, modelpolylist[loop].Vertex[1].v);
            glVertex3fv(&modelpolylist[loop].Vertex[1].x);
            glTexCoord2f(modelpolylist[loop].Vertex[2].u, modelpolylist[loop].Vertex[2].v);
            glVertex3fv(&modelpolylist[loop].Vertex[2].x);
        glEnd();
    }
    glEnable(GL_TEXTURE_2D);
    glPopMatrix();
}

void BULLET::Delete()
{
    delete[] modelvertlist;
    delete[] modelpolylist;
}

void InitializeBullets()
{
    for (int loop = 0; loop < numBullets; loop++)
    {
        bullet[loop].numverts = 8;
        bullet[loop].numpolys = 12;
        bullet[loop].Initialize();
        bullet[loop].active = 0;

        vertex[0].x = -0.05;
        vertex[0].y = 0.05;
        vertex[0].z = 0.0;

        vertex[1].x = 0.05;
        vertex[1].y = 0.05;
        vertex[1].z = 0.0;

        vertex[2].x = 0.05;
        vertex[2].y = 0.05;
        vertex[2].z = -0.2;

        vertex[3].x = -0.05;
        vertex[3].y = 0.05;
        vertex[3].z = -0.2;

        vertex[4].x = -0.05;
        vertex[4].y = -0.05;
        vertex[4].z = 0.0;

        vertex[5].x = 0.05;
        vertex[5].y = -0.05;
        vertex[5].z = 0.0;

        vertex[6].x = 0.05;
        vertex[6].y = -0.05;
        vertex[6].z = -0.2;

        vertex[7].x = -0.05;
        vertex[7].y = -0.05;
        vertex[7].z = -0.2;

        bullet[loop].modelpolylist[0].Scale[0] = 1.0;
        bullet[loop].modelpolylist[0].Scale[1] = 1.0;
        bullet[loop].modelpolylist[0].Shift[0] = 0.0;
        bullet[loop].modelpolylist[0].Shift[1] = 0.0;
        bullet[loop].modelpolylist[0].Rotate = 0.0;
        bullet[loop].modelpolylist[0].Vertex[0] = vertex[0];
        bullet[loop].modelpolylist[0].Vertex[1] = vertex[2];
        bullet[loop].modelpolylist[0].Vertex[2] = vertex[3];
        bullet[loop].modelpolylist[0].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[0].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[0].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[0].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[0].Vertex[1].v = 1.0;
        bullet[loop].modelpolylist[0].Vertex[2].u = 0.0;
        bullet[loop].modelpolylist[0].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[1].Scale[0] = 1.0;
        bullet[loop].modelpolylist[1].Scale[1] = 1.0;
        bullet[loop].modelpolylist[1].Shift[0] = 0.0;
        bullet[loop].modelpolylist[1].Shift[1] = 0.0;
        bullet[loop].modelpolylist[1].Rotate = 0.0;
        bullet[loop].modelpolylist[1].Vertex[0] = vertex[0];
        bullet[loop].modelpolylist[1].Vertex[1] = vertex[1];
        bullet[loop].modelpolylist[1].Vertex[2] = vertex[2];
        bullet[loop].modelpolylist[1].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[1].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[1].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[1].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[1].Vertex[1].v = 0.0;
        bullet[loop].modelpolylist[1].Vertex[2].u = 1.0;
        bullet[loop].modelpolylist[1].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[2].Scale[0] = 1.0;
        bullet[loop].modelpolylist[2].Scale[1] = 1.0;
        bullet[loop].modelpolylist[2].Shift[0] = 0.0;
        bullet[loop].modelpolylist[2].Shift[1] = 0.0;
        bullet[loop].modelpolylist[2].Rotate = 0.0;
        bullet[loop].modelpolylist[2].Vertex[0] = vertex[5];
        bullet[loop].modelpolylist[2].Vertex[1] = vertex[7];
        bullet[loop].modelpolylist[2].Vertex[2] = vertex[6];
        bullet[loop].modelpolylist[2].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[2].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[2].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[2].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[2].Vertex[1].v = 1.0;
        bullet[loop].modelpolylist[2].Vertex[2].u = 0.0;
        bullet[loop].modelpolylist[2].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[3].Scale[0] = 1.0;
        bullet[loop].modelpolylist[3].Scale[1] = 1.0;
        bullet[loop].modelpolylist[3].Shift[0] = 0.0;
        bullet[loop].modelpolylist[3].Shift[1] = 0.0;
        bullet[loop].modelpolylist[3].Rotate = 0.0;
        bullet[loop].modelpolylist[3].Vertex[0] = vertex[5];
        bullet[loop].modelpolylist[3].Vertex[1] = vertex[4];
        bullet[loop].modelpolylist[3].Vertex[2] = vertex[7];
        bullet[loop].modelpolylist[3].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[3].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[3].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[3].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[3].Vertex[1].v = 0.0;
        bullet[loop].modelpolylist[3].Vertex[2].u = 1.0;
        bullet[loop].modelpolylist[3].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[4].Scale[0] = 1.0;
        bullet[loop].modelpolylist[4].Scale[1] = 1.0;
        bullet[loop].modelpolylist[4].Shift[0] = 0.0;
        bullet[loop].modelpolylist[4].Shift[1] = 0.0;
        bullet[loop].modelpolylist[4].Rotate = 0.0;
        bullet[loop].modelpolylist[4].Vertex[0] = vertex[4];
        bullet[loop].modelpolylist[4].Vertex[1] = vertex[1];
        bullet[loop].modelpolylist[4].Vertex[2] = vertex[0];
        bullet[loop].modelpolylist[4].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[4].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[4].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[4].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[4].Vertex[1].v = 1.0;
        bullet[loop].modelpolylist[4].Vertex[2].u = 0.0;
        bullet[loop].modelpolylist[4].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[5].Scale[0] = 1.0;
        bullet[loop].modelpolylist[5].Scale[1] = 1.0;
        bullet[loop].modelpolylist[5].Shift[0] = 0.0;
        bullet[loop].modelpolylist[5].Shift[1] = 0.0;
        bullet[loop].modelpolylist[5].Rotate = 0.0;
        bullet[loop].modelpolylist[5].Vertex[0] = vertex[4];
        bullet[loop].modelpolylist[5].Vertex[1] = vertex[5];
        bullet[loop].modelpolylist[5].Vertex[2] = vertex[1];
        bullet[loop].modelpolylist[5].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[5].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[5].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[5].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[5].Vertex[1].v = 0.0;
        bullet[loop].modelpolylist[5].Vertex[2].u = 1.0;
        bullet[loop].modelpolylist[5].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[6].Scale[0] = 1.0;
        bullet[loop].modelpolylist[6].Scale[1] = 1.0;
        bullet[loop].modelpolylist[6].Shift[0] = 0.0;
        bullet[loop].modelpolylist[6].Shift[1] = 0.0;
        bullet[loop].modelpolylist[6].Rotate = 0.0;
        bullet[loop].modelpolylist[6].Vertex[0] = vertex[6];
        bullet[loop].modelpolylist[6].Vertex[1] = vertex[3];
        bullet[loop].modelpolylist[6].Vertex[2] = vertex[2];
        bullet[loop].modelpolylist[6].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[6].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[6].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[6].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[6].Vertex[1].v = 1.0;
        bullet[loop].modelpolylist[6].Vertex[2].u = 0.0;
        bullet[loop].modelpolylist[6].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[7].Scale[0] = 1.0;
        bullet[loop].modelpolylist[7].Scale[1] = 1.0;
        bullet[loop].modelpolylist[7].Shift[0] = 0.0;
        bullet[loop].modelpolylist[7].Shift[1] = 0.0;
        bullet[loop].modelpolylist[7].Rotate = 0.0;
        bullet[loop].modelpolylist[7].Vertex[0] = vertex[6];
        bullet[loop].modelpolylist[7].Vertex[1] = vertex[7];
        bullet[loop].modelpolylist[7].Vertex[2] = vertex[3];
        bullet[loop].modelpolylist[7].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[7].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[7].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[7].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[7].Vertex[1].v = 0.0;
        bullet[loop].modelpolylist[7].Vertex[2].u = 1.0;
        bullet[loop].modelpolylist[7].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[8].Scale[0] = 1.0;
        bullet[loop].modelpolylist[8].Scale[1] = 1.0;
        bullet[loop].modelpolylist[8].Shift[0] = 0.0;
        bullet[loop].modelpolylist[8].Shift[1] = 0.0;
        bullet[loop].modelpolylist[8].Rotate = 0.0;
        bullet[loop].modelpolylist[8].Vertex[0] = vertex[7];
        bullet[loop].modelpolylist[8].Vertex[1] = vertex[0];
        bullet[loop].modelpolylist[8].Vertex[2] = vertex[3];
        bullet[loop].modelpolylist[8].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[8].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[8].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[8].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[8].Vertex[1].v = 1.0;
        bullet[loop].modelpolylist[8].Vertex[2].u = 0.0;
        bullet[loop].modelpolylist[8].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[9].Scale[0] = 1.0;
        bullet[loop].modelpolylist[9].Scale[1] = 1.0;
        bullet[loop].modelpolylist[9].Shift[0] = 0.0;
        bullet[loop].modelpolylist[9].Shift[1] = 0.0;
        bullet[loop].modelpolylist[9].Rotate = 0.0;
        bullet[loop].modelpolylist[9].Vertex[0] = vertex[7];
        bullet[loop].modelpolylist[9].Vertex[1] = vertex[4];
        bullet[loop].modelpolylist[9].Vertex[2] = vertex[0];
        bullet[loop].modelpolylist[9].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[9].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[9].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[9].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[9].Vertex[1].v = 0.0;
        bullet[loop].modelpolylist[9].Vertex[2].u = 1.0;
        bullet[loop].modelpolylist[9].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[10].Scale[0] = 1.0;
        bullet[loop].modelpolylist[10].Scale[1] = 1.0;
        bullet[loop].modelpolylist[10].Shift[0] = 0.0;
        bullet[loop].modelpolylist[10].Shift[1] = 0.0;
        bullet[loop].modelpolylist[10].Rotate = 0.0;
        bullet[loop].modelpolylist[10].Vertex[0] = vertex[5];
        bullet[loop].modelpolylist[10].Vertex[1] = vertex[2];
        bullet[loop].modelpolylist[10].Vertex[2] = vertex[1];
        bullet[loop].modelpolylist[10].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[10].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[10].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[10].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[10].Vertex[1].v = 1.0;
        bullet[loop].modelpolylist[10].Vertex[2].u = 0.0;
        bullet[loop].modelpolylist[10].Vertex[2].v = 1.0;

        bullet[loop].modelpolylist[11].Scale[0] = 1.0;
        bullet[loop].modelpolylist[11].Scale[1] = 1.0;
        bullet[loop].modelpolylist[11].Shift[0] = 0.0;
        bullet[loop].modelpolylist[11].Shift[1] = 0.0;
        bullet[loop].modelpolylist[11].Rotate = 0.0;
        bullet[loop].modelpolylist[11].Vertex[0] = vertex[5];
        bullet[loop].modelpolylist[11].Vertex[1] = vertex[6];
        bullet[loop].modelpolylist[11].Vertex[2] = vertex[2];
        bullet[loop].modelpolylist[11].Texture = texture[4].TexID;
        bullet[loop].modelpolylist[11].Vertex[0].u = 0.0;
        bullet[loop].modelpolylist[11].Vertex[0].v = 0.0;
        bullet[loop].modelpolylist[11].Vertex[1].u = 1.0;
        bullet[loop].modelpolylist[11].Vertex[1].v = 0.0;
        bullet[loop].modelpolylist[11].Vertex[2].u = 1.0;
        bullet[loop].modelpolylist[11].Vertex[2].v = 1.0;
    }
}

void UpdateBullets()
{
    // Update Bullets
    int Pos, CollisionNode, CollisionPoly;
    VECTOR EndPos, p0, pN, a, b, c;
    ListNode* tempListNode;
    BSP_node* tempNode;
    DECAL* tempDecal;
    float magnitudedistance, shortestdistance;
    VECTOR distance, CollisionPoint;
    // Loop through maximum number of bullets
    for (int loop = 0; loop < numBullets; loop++)
    {
        // Set to a distance further than any in the map
        shortestdistance = 1000000.0;
        // If the bullet is active
        if (bullet[loop].active == true)
        {
            // If it has just been fired
            if (bullet[loop].flag == true)
            {
                // Loop through all leaves
                for (CollisionNode = 1; CollisionNode <= numleaves; CollisionNode++)
                {
                    // Get the leaf node
                    tempListNode = LeafList.Get(CollisionNode);
                    tempNode = tempListNode->node;
                    // If the node is marked as visible
                    if (tempNode->visible == true)
                    {
                        // Loop through all polygons in the leaf
                        for (CollisionPoly = 0; CollisionPoly < tempNode->numpolys; CollisionPoly++)
                        {
                            // Get a point on the polygon
                            p0.x = tempNode->nodepolylist[CollisionPoly].Vertex[0].x;
                            p0.y = tempNode->nodepolylist[CollisionPoly].Vertex[0].y;
                            p0.z = tempNode->nodepolylist[CollisionPoly].Vertex[0].z;
                            // Get the normal of the polygon
                            pN.x = tempNode->nodepolylist[CollisionPoly].Vertex[0].nx;
                            pN.y = tempNode->nodepolylist[CollisionPoly].Vertex[0].ny;
                            pN.z = tempNode->nodepolylist[CollisionPoly].Vertex[0].nz;
                            // If the bullet isn't in front of the polygon then continue
                            if (ClassifyPoint(bullet[loop].Position, p0, pN) != 1)
                                continue;
                            // Find the bullets final position
                            EndPos.x = bullet[loop].Position.x - bullet[loop].VelocityVector.x * 500.0;
                            EndPos.y = bullet[loop].Position.y - bullet[loop].VelocityVector.y * 500.0;
                            EndPos.z = bullet[loop].Position.z - bullet[loop].VelocityVector.z * 500.0;
                            // Find collision point on the plane
                            CollisionPoint = line_plane_collision(&bullet[loop].Position, &EndPos, &tempNode->nodepolylist[CollisionPoly]);
                            // If the collision point is outside the frustum then continue (stops collisions behind the player)
                            if (!CheckClipPlanes(camera[currentCamera], CollisionPoint))
                                continue;
                            a.x = tempNode->nodepolylist[CollisionPoly].Vertex[0].x;
                            a.y = tempNode->nodepolylist[CollisionPoly].Vertex[0].y;
                            a.z = tempNode->nodepolylist[CollisionPoly].Vertex[0].z;
                            b.x = tempNode->nodepolylist[CollisionPoly].Vertex[1].x;
                            b.y = tempNode->nodepolylist[CollisionPoly].Vertex[1].y;
                            b.z = tempNode->nodepolylist[CollisionPoly].Vertex[1].z;
                            c.x = tempNode->nodepolylist[CollisionPoly].Vertex[2].x;
                            c.y = tempNode->nodepolylist[CollisionPoly].Vertex[2].y;
                            c.z = tempNode->nodepolylist[CollisionPoly].Vertex[2].z;
                            // If collision point is within the polygon
                            if (CheckPointInTriangle(CollisionPoint, a, b, c))
                            {
                                // Get the collision distance
                                distance = CollisionPoint - bullet[loop].Position;
                                magnitudedistance = distance.GetMagnitude();
                                // If collision distance is shorter than previous distance
                                if (magnitudedistance < shortestdistance)
                                {
                                    // Bullet has been processed so set first time flag to false
                                    bullet[loop].flag = false;
                                    // Set new shortest distance
                                    shortestdistance = magnitudedistance;
                                    // Set collision details
                                    bullet[loop].CollisionVector = CollisionPoint;
                                    bullet[loop].collisionnode = CollisionNode;
                                    bullet[loop].collisionpoly = CollisionPoly;
                                }
                            }
                        }
                    }
                }
            }
            // Set bullet speed
            float bulletFactor;
            if (bulletspeed == 0)
                bulletFactor = 1200.0;
            else
                bulletFactor = 10.0;
            // Set the previous position
            bullet[loop].PreviousPosition = bullet[loop].Position;
            // Get the present position
            bullet[loop].Position.x -= bullet[loop].VelocityVector.x * bulletFactor * bullet[loop].Multiplier;
            bullet[loop].Position.y -= bullet[loop].VelocityVector.y * bulletFactor * bullet[loop].Multiplier;
            bullet[loop].Position.z -= bullet[loop].VelocityVector.z * bulletFactor * bullet[loop].Multiplier;
            // Find the leaf node of the bullets position (to determine when to render it)
            bullet[loop].leaf = FindCurrentLeaf(bullet[loop].Position, root);
            // Get the bullets collision node
            tempListNode = LeafList.Get(bullet[loop].collisionnode);
            tempNode = tempListNode->node;
            // Get a point on the collision polygon and it's normal
            p0.x = tempNode->nodepolylist[bullet[loop].collisionpoly].Vertex[0].x;
            p0.y = tempNode->nodepolylist[bullet[loop].collisionpoly].Vertex[0].y;
            p0.z = tempNode->nodepolylist[bullet[loop].collisionpoly].Vertex[0].z;
            pN.x = tempNode->nodepolylist[bullet[loop].collisionpoly].Vertex[0].nx;
            pN.y = tempNode->nodepolylist[bullet[loop].collisionpoly].Vertex[0].ny;
            pN.z = tempNode->nodepolylist[bullet[loop].collisionpoly].Vertex[0].nz;
            // Classify the bullet position with respect to the collision polygon
            Pos = ClassifyPoint(bullet[loop].Position, p0, pN);
            // If the bullet is on or behind the collision polygon
            if (Pos != 1)
            {
                // Set the bullet to inactive
                bullet[loop].active = false;
                // Decrease the active bullet count
                numBulletsActive--;
                // Create the particles
                VECTOR CollisionPoint = line_plane_collision(&bullet[loop].PreviousPosition, &bullet[loop].Position, &tempNode->nodepolylist[bullet[loop].collisionpoly]);
                CreateSparks(bullet[loop].PreviousPosition, CollisionPoint, pN);
                // If we have not reached the maximum decals for this node
                if (tempNode->numdecals < maxDecals)
                {
                    // Create a new decal and set it's properties
                    tempNode->numdecals++;
                    tempDecal = new DECAL;
                    // Set the decal to the current global type of decal selected
                    tempDecal->type = bullettype;
                    // Set the decal's size accordingly
                    if (tempDecal->type == 0)
                        tempDecal->Size = 0.5;
                    else
                        tempDecal->Size = 1.0;
                    tempDecal->linkPosition = tempNode->numdecals;
                    tempDecal->CollisionVector = bullet[loop].CollisionVector;
                    // Set persistence counter (decreased with each render)
                    tempDecal->counter = 2000;

                    CreateDecal(tempDecal->CollisionVector, &tempNode->nodepolylist[bullet[loop].collisionpoly], tempDecal);
                    tempNode->decallist.Insert(tempDecal);

                    // Set impact-flash decal properties
                    impactflashdecal.CollisionVector = bullet[loop].CollisionVector;
                    CreateDecal(impactflashdecal.CollisionVector, &tempNode->nodepolylist[bullet[loop].collisionpoly], &impactflashdecal);
                    impactflashdecal.active = true;
                    impactflashdecal.Size = 1;
                }
            }
            // Find distance of bullet from camera
            distance = bullet[loop].Position - camera[currentCamera].Position;
            magnitudedistance = distance.GetMagnitude();
            // If the distance is greater than 500 units then make bullet inactive
            if (magnitudedistance > 500)
            {
                bullet[loop].active = 0;
                numBulletsActive--;
            }
        }
    }
}

void DeleteBullets()
{
    // Call the delete method for all bullets
    for (int loop = 0; loop < numBullets; loop++)
        bullet[loop].Delete();
    // Delete the array of bullets
    delete[] bullet;
}
