#include <windows.h>
#include "shared.h"
#include "polygon.h"
#include "camera.h"
#include "texture.h"
#include "light.h"
#include "locmath.h"
#include "bspline.h"
#include "glFont.h"
#include "bsp.h"
#include "tll.h"
#include "plane.h"


extern int numPolygons;
extern int numCameras;
extern int cameraMode;
extern int currentCamera;
extern int numLights;
extern int currentLight;
extern int fps;
extern float multiplier;
extern GLFONT myFont;
extern char HaloTypeName[MAX_PATH];
extern char FireTypeName[MAX_PATH];
extern int numSplines;
extern int lookAtPath;
extern int currentSpline;
extern SPLINE* spline;
extern char SplineFileName[MAX_PATH];
extern PLANE frustum[6];
extern int showportals;
extern int currentleaf;
extern int numcurrentportals;
extern int numleavesvisible;

//Portion Copyright (C) Kevin Kaiser, 2000
VECTOR line_plane_collision(VECTOR *a, VECTOR *b, POLYGON *tri1)
{
    float Distance = -(tri1->Vertex[0].x * tri1->Vertex[0].nx + tri1->Vertex[0].y * tri1->Vertex[0].ny + tri1->Vertex[0].z * tri1->Vertex[0].nz);
    float final_x,final_y,final_z,final_t;
    float t,i;
    VECTOR temp;

    t=(float)0; i=(float)0;
    i+=(float)(tri1->Vertex[0].nx*b->x)+(tri1->Vertex[0].ny*b->y)+(tri1->Vertex[0].nz*b->z)+(Distance);
    t+=(float)(tri1->Vertex[0].nx*(b->x*-1))+(tri1->Vertex[0].ny*(b->y*-1))+(tri1->Vertex[0].nz*(b->z*-1));
    t+=(float)(tri1->Vertex[0].nx*a->x)+(tri1->Vertex[0].ny*a->y)+(tri1->Vertex[0].nz*a->z);

    // Be wary of possible divide-by-zeros here (i.e. if t==0)
    final_t = (-i)/t;

    // Vertical Line Segment
    if ((a->x == b->x)&&(a->z == b->z))
    { // vertical line segment

        temp.x = a->x;
        temp.y = (-((tri1->Vertex[0].nx*a->x)+(tri1->Vertex[0].nz*a->z)+(Distance)))/(tri1->Vertex[0].ny);
        temp.z = a->z;

        return(temp);
    }

    final_x = (((a->x)*(final_t))+((b->x)*(1-final_t)));
    final_y = (((a->y)*(final_t))+((b->y)*(1-final_t)));
    final_z = (((a->z)*(final_t))+((b->z)*(1-final_t)));

    temp.x = final_x;
    temp.y = final_y;
    temp.z = final_z;

    return(temp);
}

int SphereInFrustum(VECTOR point, float radius )
{
   int p;
   int c = 0;
   float d;

   for( p = 0; p < 6; p++ )
   {
      d = frustum[p].nx * point.x + frustum[p].ny * point.y + frustum[p].nz * point.z + frustum[p].Distance;
      if( d <= -radius )
         return 0;
      if( d > radius )
         c++;
   }
   return (c == 6) ? 2 : 1;
}

bool PointInFrustum(VECTOR point)
{
   int p;

   for( p = 0; p < 6; p++ )
      if( frustum[p].nx * point.x + frustum[p].ny * point.y + frustum[p].nz * point.z + frustum[p].Distance <= 0 )
         return false;
   return true;
}

void ExtractFrustum()
{
    float proj[16];
    float modl[16];
    float clip[16];
    float t;
    /* Get the current PROJECTION matrix from OpenGL */
    glGetFloatv( GL_PROJECTION_MATRIX, proj );
    /* Get the current MODELVIEW matrix from OpenGL */
    glGetFloatv( GL_MODELVIEW_MATRIX, modl );
    /* Combine the two matrices (multiply projection by modelview) */
    clip[ 0] = modl[ 0] * proj[ 0] + modl[ 1] * proj[ 4] + modl[ 2] * proj[ 8] + modl[ 3] * proj[12];
    clip[ 1] = modl[ 0] * proj[ 1] + modl[ 1] * proj[ 5] + modl[ 2] * proj[ 9] + modl[ 3] * proj[13];
    clip[ 2] = modl[ 0] * proj[ 2] + modl[ 1] * proj[ 6] + modl[ 2] * proj[10] + modl[ 3] * proj[14];
    clip[ 3] = modl[ 0] * proj[ 3] + modl[ 1] * proj[ 7] + modl[ 2] * proj[11] + modl[ 3] * proj[15];
    clip[ 4] = modl[ 4] * proj[ 0] + modl[ 5] * proj[ 4] + modl[ 6] * proj[ 8] + modl[ 7] * proj[12];
    clip[ 5] = modl[ 4] * proj[ 1] + modl[ 5] * proj[ 5] + modl[ 6] * proj[ 9] + modl[ 7] * proj[13];
    clip[ 6] = modl[ 4] * proj[ 2] + modl[ 5] * proj[ 6] + modl[ 6] * proj[10] + modl[ 7] * proj[14];
    clip[ 7] = modl[ 4] * proj[ 3] + modl[ 5] * proj[ 7] + modl[ 6] * proj[11] + modl[ 7] * proj[15];
    clip[ 8] = modl[ 8] * proj[ 0] + modl[ 9] * proj[ 4] + modl[10] * proj[ 8] + modl[11] * proj[12];
    clip[ 9] = modl[ 8] * proj[ 1] + modl[ 9] * proj[ 5] + modl[10] * proj[ 9] + modl[11] * proj[13];
    clip[10] = modl[ 8] * proj[ 2] + modl[ 9] * proj[ 6] + modl[10] * proj[10] + modl[11] * proj[14];
    clip[11] = modl[ 8] * proj[ 3] + modl[ 9] * proj[ 7] + modl[10] * proj[11] + modl[11] * proj[15];
    clip[12] = modl[12] * proj[ 0] + modl[13] * proj[ 4] + modl[14] * proj[ 8] + modl[15] * proj[12];
    clip[13] = modl[12] * proj[ 1] + modl[13] * proj[ 5] + modl[14] * proj[ 9] + modl[15] * proj[13];
    clip[14] = modl[12] * proj[ 2] + modl[13] * proj[ 6] + modl[14] * proj[10] + modl[15] * proj[14];
    clip[15] = modl[12] * proj[ 3] + modl[13] * proj[ 7] + modl[14] * proj[11] + modl[15] * proj[15];

    /* Extract the numbers for the RIGHT plane */
    frustum[0].nx = clip[ 3] - clip[ 0];
    frustum[0].ny = clip[ 7] - clip[ 4];
    frustum[0].nz = clip[11] - clip[ 8];
    frustum[0].Distance = clip[15] - clip[12];
    /* Normalize the result */
    t = sqrt( frustum[0].nx * frustum[0].nx + frustum[0].ny * frustum[0].ny + frustum[0].nz * frustum[0].nz);
    frustum[0].nx /= t;
    frustum[0].ny /= t;
    frustum[0].nz /= t;
    frustum[0].Distance /= t;
    /* Extract the numbers for the LEFT plane */
    frustum[1].nx = clip[ 3] + clip[ 0];
    frustum[1].ny = clip[ 7] + clip[ 4];
    frustum[1].nz = clip[11] + clip[ 8];
    frustum[1].Distance = clip[15] + clip[12];
    /* Normalize the result */
    t = sqrt( frustum[1].nx * frustum[1].nx + frustum[1].ny * frustum[1].ny + frustum[1].nz * frustum[1].nz);
    frustum[1].nx /= t;
    frustum[1].ny /= t;
    frustum[1].nz /= t;
    frustum[1].Distance /= t;
    /* Extract the BOTTOM plane */
    frustum[2].nx = clip[ 3] + clip[ 1];
    frustum[2].ny = clip[ 7] + clip[ 5];
    frustum[2].nz = clip[11] + clip[ 9];
    frustum[2].Distance = clip[15] + clip[13];
    /* Normalize the result */
    t = sqrt( frustum[2].nx * frustum[2].nx + frustum[2].ny * frustum[2].ny + frustum[2].nz * frustum[2].nz);
    frustum[2].nx /= t;
    frustum[2].ny /= t;
    frustum[2].nz /= t;
    frustum[2].Distance /= t;
    /* Extract the TOP plane */
    frustum[3].nx = clip[ 3] - clip[ 1];
    frustum[3].ny = clip[ 7] - clip[ 5];
    frustum[3].nz = clip[11] - clip[ 9];
    frustum[3].Distance = clip[15] - clip[13];
    /* Normalize the result */
    t = sqrt( frustum[3].nx * frustum[3].nx + frustum[3].ny * frustum[3].ny + frustum[3].nz * frustum[3].nz);
    frustum[3].nx /= t;
    frustum[3].ny /= t;
    frustum[3].nz /= t;
    frustum[3].Distance /= t;
    /* Extract the FAR plane */
    frustum[4].nx = clip[ 3] - clip[ 2];
    frustum[4].ny = clip[ 7] - clip[ 6];
    frustum[4].nz = clip[11] - clip[10];
    frustum[4].Distance = clip[15] - clip[14];
    /* Normalize the result */
    t = sqrt( frustum[4].nx * frustum[4].nx + frustum[4].ny * frustum[4].ny + frustum[4].nz * frustum[4].nz);
    frustum[4].nx /= t;
    frustum[4].ny /= t;
    frustum[4].nz /= t;
    frustum[4].Distance /= t;
    /* Extract the NEAR plane */
    frustum[5].nx = clip[ 3] + clip[ 2];
    frustum[5].ny = clip[ 7] + clip[ 6];
    frustum[5].nz = clip[11] + clip[10];
    frustum[5].Distance = clip[15] + clip[14];
    /* Normalize the result */
    t = sqrt( frustum[5].nx * frustum[5].nx + frustum[5].ny * frustum[5].ny + frustum[5].nz * frustum[5].nz);
    frustum[5].nx /= t;
    frustum[5].ny /= t;
    frustum[5].nz /= t;
    frustum[5].Distance /= t;
}

void AddSpline(int Number, LinkedList<SPLINE>& SplineList)
{
    SPLINE* spline2 = new SPLINE;
    spline2->Active = 1;
    spline2->Repeat = 1;
    spline2->Degree = 3;
    spline2->NumControl = 7;
    spline2->NumPoints = 100;
    spline2->Control = new VECTOR[100];
    spline2->Output = new VECTOR[1000];
    spline2->StartTime = 5000;
    spline2->EndTime = 25000;
    spline2->CopyOfStartTime = spline2->StartTime;
    spline2->CopyOfEndTime = spline2->EndTime;

    spline2->Red = ((float)(rand()%226) + 30.0) / 255;
    spline2->Green = ((float)(rand()%226) + 30.0) / 255;
    spline2->Blue = ((float)(rand()%266) + 30.0) / 255;

    for (int loop = 0; loop < 100; loop++)
    {
        spline2->Control[loop].x = rand()%60 - 29;  spline2->Control[loop].y = rand()%60 - 29;  spline2->Control[loop].z = rand()%60 - 29;
    }

    spline2->linkPosition = Number;   //Set the link position of the spline
    SplineList.Insert(spline2);       //Insert spline in linked list
}

void DeleteSpline(int Number, LinkedList<SPLINE>& SplineList)
{
    SPLINE* temp = SplineList.Get(Number);
    delete[] temp->Control;
    delete[] temp->Output;
    SplineList.Delete(Number);
    delete temp;
}

int LoadSplines(char* SplineFileName, LinkedList<SPLINE>& SplineList)
{
    const int stringLength = 33;
    char tempString[stringLength];
    FILE* SplineFile;
    SplineFile = fopen(SplineFileName, "rt");
    if (!SplineFile)
    {
        MessageBox(NULL, "Spline File didn't open", "Error", MB_OK | MB_ICONERROR);
        return 0;
    }
    fseek(SplineFile, 0, SEEK_SET);
    int InitialNumSplines = numSplines;
    if (!fgets(tempString, stringLength, SplineFile))
    {
        MessageBox(NULL, "Error reading from the spline data file", "Error", MB_OK | MB_ICONERROR);
        numSplines = InitialNumSplines;
        return FALSE;
    }
    numSplines = atoi(tempString);
    if (InitialNumSplines > numSplines)
    {
        for (int loop = InitialNumSplines - 1; loop >= numSplines; loop--)
        {
            DeleteSpline(loop, SplineList);
        }
    }
    if (numSplines > InitialNumSplines)
    {
        for (int loop = InitialNumSplines; loop < numSplines; loop++)
        {
            AddSpline(loop, SplineList);
        }
    }
    for (int loop = 0; loop < numSplines; loop++)
    {
        spline = SplineList.Get(loop);
        // Active flag
        fgets(tempString, stringLength, SplineFile);
        spline->Active = atoi(tempString);
        // Repeat flag
        fgets(tempString, stringLength, SplineFile);
        spline->Repeat = atoi(tempString);
        // Degree
        fgets(tempString, stringLength, SplineFile);
        spline->Degree = atoi(tempString);
        // NumControl
        fgets(tempString, stringLength, SplineFile);
        spline->NumControl = atoi(tempString);
        // NumPoints
        fgets(tempString, stringLength, SplineFile);
        spline->NumPoints = atoi(tempString);
        // StartTime
        fgets(tempString, stringLength, SplineFile);
        spline->StartTime = atof(tempString);
        // EndTime
        fgets(tempString, stringLength, SplineFile);
        spline->EndTime = atof(tempString);
        // Red
        fgets(tempString, stringLength, SplineFile);
        spline->Red = atof(tempString);
        // Green
        fgets(tempString, stringLength, SplineFile);
        spline->Green = atof(tempString);
        // Blue
        fgets(tempString, stringLength, SplineFile);
        spline->Blue = atof(tempString);

        for (int loop2 = 0; loop2 <= spline->NumControl; loop2++)
        {
            fgets(tempString, stringLength, SplineFile);
            spline->Control[loop2].x = atof(tempString);
            fgets(tempString, stringLength, SplineFile);
            spline->Control[loop2].y = atof(tempString);
            fgets(tempString, stringLength, SplineFile);
            spline->Control[loop2].z = atof(tempString);
        }
        spline->CopyOfStartTime = spline->StartTime;
        spline->CopyOfEndTime = spline->EndTime;
    }

    currentSpline = numSplines - 1;
    if (lookAtPath >= numSplines)
        lookAtPath = numSplines - 1;
    if (fclose(SplineFile))
        MessageBox(NULL, "File didn't close", "Error", MB_OK | MB_ICONERROR);
    return 1;
}

void SetSplines(LinkedList<SPLINE>& SplineList)
{
    srand((unsigned)time(NULL));
    //Create a new spline
    for (int loop = 0; loop < numSplines; loop++)
    {
        AddSpline(loop, SplineList);
    }
    //Load splines from data file
    LoadSplines(SplineFileName, SplineList);
}

void SetGLLighting(LIGHT* light)         // Loop through and reset all the lights
{
    int temp;
    for (temp = 0; temp <= numLights; temp++)
    {
        light[temp].LightNumber = temp;
        light[temp].Reset();
    }
    glEnable(GL_LIGHTING);
}

void SetGLCamera(CAMERA* camera)          // Loop through and reset all the cameras
{
    int temp;
    for(temp = 0; temp <= numCameras; temp++)
        camera[temp].Reset();
}

void SetGLVertices(VERTEX* vertex)
{
    vertex[0].x = -80;
    vertex[0].y = -10;
    vertex[0].z = -60;

    vertex[1].x = 20;
    vertex[1].y = -10;
    vertex[1].z = -60;

    vertex[2].x = 20;
    vertex[2].y = -10;
    vertex[2].z = -40;

    vertex[3].x = 170;
    vertex[3].y = -10;
    vertex[3].z = -40;

    vertex[4].x = 170;
    vertex[4].y = -10;
    vertex[4].z = 40;

    vertex[5].x = 120;
    vertex[5].y = -10;
    vertex[5].z = 40;

    vertex[6].x = 120;
    vertex[6].y = -10;
    vertex[6].z = 80;

    vertex[7].x = 160;
    vertex[7].y = -10;
    vertex[7].z = 80;

    vertex[8].x = 160;
    vertex[8].y = -10;
    vertex[8].z = 160;

    vertex[9].x = 60;
    vertex[9].y = -10;
    vertex[9].z = 160;

    vertex[10].x = 60;
    vertex[10].y = -10;
    vertex[10].z = 80;

    vertex[11].x = 100;
    vertex[11].y = -10;
    vertex[11].z = 80;

    vertex[12].x = 100;
    vertex[12].y = -10;
    vertex[12].z = 40;

    vertex[13].x = 20;
    vertex[13].y = -10;
    vertex[13].z = 40;

    vertex[14].x = 20;
    vertex[14].y = -10;
    vertex[14].z = 60;

    vertex[15].x = -80;
    vertex[15].y = -10;
    vertex[15].z = 60;

    vertex[16].x = -80;
    vertex[16].y = 10;
    vertex[16].z = -60;

    vertex[17].x = 20;
    vertex[17].y = 10;
    vertex[17].z = -60;

    vertex[18].x = 20;
    vertex[18].y = 10;
    vertex[18].z = -40;

    vertex[19].x = 170;
    vertex[19].y = 10;
    vertex[19].z = -40;

    vertex[20].x = 170;
    vertex[20].y = 10;
    vertex[20].z = 40;

    vertex[21].x = 120;
    vertex[21].y = 10;
    vertex[21].z = 40;

    vertex[22].x = 120;
    vertex[22].y = 10;
    vertex[22].z = 80;

    vertex[23].x = 160;
    vertex[23].y = 10;
    vertex[23].z = 80;

    vertex[24].x = 160;
    vertex[24].y = 10;
    vertex[24].z = 160;

    vertex[25].x = 60;
    vertex[25].y = 10;
    vertex[25].z = 160;

    vertex[26].x = 60;
    vertex[26].y = 10;
    vertex[26].z = 80;

    vertex[27].x = 100;
    vertex[27].y = 10;
    vertex[27].z = 80;

    vertex[28].x = 100;
    vertex[28].y = 10;
    vertex[28].z = 40;

    vertex[29].x = 20;
    vertex[29].y = 10;
    vertex[29].z = 40;

    vertex[30].x = 20;
    vertex[30].y = 10;
    vertex[30].z = 60;

    vertex[31].x = -80;
    vertex[31].y = 10;
    vertex[31].z = 60;

    vertex[32].x = -80;
    vertex[32].y = 10;
    vertex[32].z = 40;

    vertex[33].x = -80;
    vertex[33].y = 40;
    vertex[33].z = 0;

    vertex[34].x = -80;
    vertex[34].y = 10;
    vertex[34].z = -40;

    vertex[35].x = 20;
    vertex[35].y = 40;
    vertex[35].z = 0;

    vertex[36].x = 170;
    vertex[36].y = 40;
    vertex[36].z = 0;
}

// Initialize the world data
void SetGLWorld(POLYGON* polygon, TEXTURE* texture, VERTEX* vertex)
{
    SetGLVertices(vertex);

    polygon[0].Scale[0] = 6.0;
    polygon[0].Scale[1] = 1.0;
    polygon[0].Shift[0] = 0.0;
    polygon[0].Shift[1] = 0.0;
    polygon[0].Rotate = 0.0;
    polygon[0].Vertex[0] = vertex[0];
    polygon[0].Vertex[1] = vertex[17];
    polygon[0].Vertex[2] = vertex[16];
    polygon[0].Texture = texture[2].TexID;
    polygon[0].Vertex[0].u = 0.0;
    polygon[0].Vertex[0].v = 0.0;
    polygon[0].Vertex[1].u = 1.0;
    polygon[0].Vertex[1].v = 1.0;
    polygon[0].Vertex[2].u = 0.0;
    polygon[0].Vertex[2].v = 1.0;

    polygon[1].Scale[0] = 6.0;
    polygon[1].Scale[1] = 1.0;
    polygon[1].Shift[0] = 0.0;
    polygon[1].Shift[1] = 0.0;
    polygon[1].Rotate = 0.0;
    polygon[1].Vertex[0] = vertex[0];
    polygon[1].Vertex[1] = vertex[1];
    polygon[1].Vertex[2] = vertex[17];
    polygon[1].Texture = texture[2].TexID;
    polygon[1].Vertex[0].u = 0.0;
    polygon[1].Vertex[0].v = 0.0;
    polygon[1].Vertex[1].u = 1.0;
    polygon[1].Vertex[1].v = 0.0;
    polygon[1].Vertex[2].u = 1.0;
    polygon[1].Vertex[2].v = 1.0;

    polygon[2].Scale[0] = 1.0;
    polygon[2].Scale[1] = 1.0;
    polygon[2].Shift[0] = 0.0;
    polygon[2].Shift[1] = 0.0;
    polygon[2].Rotate = 0.0;
    polygon[2].Vertex[0] = vertex[1];
    polygon[2].Vertex[1] = vertex[18];
    polygon[2].Vertex[2] = vertex[17];
    polygon[2].Texture = texture[2].TexID;
    polygon[2].Vertex[0].u = 0.0;
    polygon[2].Vertex[0].v = 0.0;
    polygon[2].Vertex[1].u = 1.0;
    polygon[2].Vertex[1].v = 1.0;
    polygon[2].Vertex[2].u = 0.0;
    polygon[2].Vertex[2].v = 1.0;

    polygon[3].Scale[0] = 1.0;
    polygon[3].Scale[1] = 1.0;
    polygon[3].Shift[0] = 0.0;
    polygon[3].Shift[1] = 0.0;
    polygon[3].Rotate = 0.0;
    polygon[3].Vertex[0] = vertex[1];
    polygon[3].Vertex[1] = vertex[2];
    polygon[3].Vertex[2] = vertex[18];
    polygon[3].Texture = texture[2].TexID;
    polygon[3].Vertex[0].u = 0.0;
    polygon[3].Vertex[0].v = 0.0;
    polygon[3].Vertex[1].u = 1.0;
    polygon[3].Vertex[1].v = 0.0;
    polygon[3].Vertex[2].u = 1.0;
    polygon[3].Vertex[2].v = 1.0;

    polygon[4].Scale[0] = 5.0;
    polygon[4].Scale[1] = 1.0;
    polygon[4].Shift[0] = 0.0;
    polygon[4].Shift[1] = 0.0;
    polygon[4].Rotate = 0.0;
    polygon[4].Vertex[0] = vertex[2];
    polygon[4].Vertex[1] = vertex[19];
    polygon[4].Vertex[2] = vertex[18];
    polygon[4].Texture = texture[2].TexID;
    polygon[4].Vertex[0].u = 0.0;
    polygon[4].Vertex[0].v = 0.0;
    polygon[4].Vertex[1].u = 1.0;
    polygon[4].Vertex[1].v = 1.0;
    polygon[4].Vertex[2].u = 0.0;
    polygon[4].Vertex[2].v = 1.0;

    polygon[5].Scale[0] = 5.0;
    polygon[5].Scale[1] = 1.0;
    polygon[5].Shift[0] = 0.0;
    polygon[5].Shift[1] = 0.0;
    polygon[5].Rotate = 0.0;
    polygon[5].Vertex[0] = vertex[2];
    polygon[5].Vertex[1] = vertex[3];
    polygon[5].Vertex[2] = vertex[19];
    polygon[5].Texture = texture[2].TexID;
    polygon[5].Vertex[0].u = 0.0;
    polygon[5].Vertex[0].v = 0.0;
    polygon[5].Vertex[1].u = 1.0;
    polygon[5].Vertex[1].v = 0.0;
    polygon[5].Vertex[2].u = 1.0;
    polygon[5].Vertex[2].v = 1.0;

    polygon[6].Scale[0] = 4.0;
    polygon[6].Scale[1] = 1.0;
    polygon[6].Shift[0] = 0.0;
    polygon[6].Shift[1] = 0.0;
    polygon[6].Rotate = 0.0;
    polygon[6].Vertex[0] = vertex[3];
    polygon[6].Vertex[1] = vertex[20];
    polygon[6].Vertex[2] = vertex[19];
    polygon[6].Texture = texture[2].TexID;
    polygon[6].Vertex[0].u = 0.0;
    polygon[6].Vertex[0].v = 0.0;
    polygon[6].Vertex[1].u = 1.0;
    polygon[6].Vertex[1].v = 1.0;
    polygon[6].Vertex[2].u = 0.0;
    polygon[6].Vertex[2].v = 1.0;

    polygon[7].Scale[0] = 4.0;
    polygon[7].Scale[1] = 1.0;
    polygon[7].Shift[0] = 0.0;
    polygon[7].Shift[1] = 0.0;
    polygon[7].Rotate = 0.0;
    polygon[7].Vertex[0] = vertex[3];
    polygon[7].Vertex[1] = vertex[4];
    polygon[7].Vertex[2] = vertex[20];
    polygon[7].Texture = texture[2].TexID;
    polygon[7].Vertex[0].u = 0.0;
    polygon[7].Vertex[0].v = 0.0;
    polygon[7].Vertex[1].u = 1.0;
    polygon[7].Vertex[1].v = 0.0;
    polygon[7].Vertex[2].u = 1.0;
    polygon[7].Vertex[2].v = 1.0;

    polygon[8].Scale[0] = 2.0;
    polygon[8].Scale[1] = 1.0;
    polygon[8].Shift[0] = 0.0;
    polygon[8].Shift[1] = 0.0;
    polygon[8].Rotate = 0.0;
    polygon[8].Vertex[0] = vertex[4];
    polygon[8].Vertex[1] = vertex[21];
    polygon[8].Vertex[2] = vertex[20];
    polygon[8].Texture = texture[2].TexID;
    polygon[8].Vertex[0].u = 0.0;
    polygon[8].Vertex[0].v = 0.0;
    polygon[8].Vertex[1].u = 1.0;
    polygon[8].Vertex[1].v = 1.0;
    polygon[8].Vertex[2].u = 0.0;
    polygon[8].Vertex[2].v = 1.0;

    polygon[9].Scale[0] = 2.0;
    polygon[9].Scale[1] = 1.0;
    polygon[9].Shift[0] = 0.0;
    polygon[9].Shift[1] = 0.0;
    polygon[9].Rotate = 0.0;
    polygon[9].Vertex[0] = vertex[4];
    polygon[9].Vertex[1] = vertex[5];
    polygon[9].Vertex[2] = vertex[21];
    polygon[9].Texture = texture[2].TexID;
    polygon[9].Vertex[0].u = 0.0;
    polygon[9].Vertex[0].v = 0.0;
    polygon[9].Vertex[1].u = 1.0;
    polygon[9].Vertex[1].v = 0.0;
    polygon[9].Vertex[2].u = 1.0;
    polygon[9].Vertex[2].v = 1.0;

    polygon[10].Scale[0] = 2.0;
    polygon[10].Scale[1] = 1.0;
    polygon[10].Shift[0] = 0.0;
    polygon[10].Shift[1] = 0.0;
    polygon[10].Rotate = 0.0;
    polygon[10].Vertex[0] = vertex[5];
    polygon[10].Vertex[1] = vertex[22];
    polygon[10].Vertex[2] = vertex[21];
    polygon[10].Texture = texture[2].TexID;
    polygon[10].Vertex[0].u = 0.0;
    polygon[10].Vertex[0].v = 0.0;
    polygon[10].Vertex[1].u = 1.0;
    polygon[10].Vertex[1].v = 1.0;
    polygon[10].Vertex[2].u = 0.0;
    polygon[10].Vertex[2].v = 1.0;

    polygon[11].Scale[0] = 2.0;
    polygon[11].Scale[1] = 1.0;
    polygon[11].Shift[0] = 0.0;
    polygon[11].Shift[1] = 0.0;
    polygon[11].Rotate = 0.0;
    polygon[11].Vertex[0] = vertex[5];
    polygon[11].Vertex[1] = vertex[6];
    polygon[11].Vertex[2] = vertex[22];
    polygon[11].Texture = texture[2].TexID;
    polygon[11].Vertex[0].u = 0.0;
    polygon[11].Vertex[0].v = 0.0;
    polygon[11].Vertex[1].u = 1.0;
    polygon[11].Vertex[1].v = 0.0;
    polygon[11].Vertex[2].u = 1.0;
    polygon[11].Vertex[2].v = 1.0;

    polygon[12].Scale[0] = 2.0;
    polygon[12].Scale[1] = 1.0;
    polygon[12].Shift[0] = 0.0;
    polygon[12].Shift[1] = 0.0;
    polygon[12].Rotate = 0.0;
    polygon[12].Vertex[0] = vertex[6];
    polygon[12].Vertex[1] = vertex[23];
    polygon[12].Vertex[2] = vertex[22];
    polygon[12].Texture = texture[2].TexID;
    polygon[12].Vertex[0].u = 0.0;
    polygon[12].Vertex[0].v = 0.0;
    polygon[12].Vertex[1].u = 1.0;
    polygon[12].Vertex[1].v = 1.0;
    polygon[12].Vertex[2].u = 0.0;
    polygon[12].Vertex[2].v = 1.0;

    polygon[13].Scale[0] = 2.0;
    polygon[13].Scale[1] = 1.0;
    polygon[13].Shift[0] = 0.0;
    polygon[13].Shift[1] = 0.0;
    polygon[13].Rotate = 0.0;
    polygon[13].Vertex[0] = vertex[6];
    polygon[13].Vertex[1] = vertex[7];
    polygon[13].Vertex[2] = vertex[23];
    polygon[13].Texture = texture[2].TexID;
    polygon[13].Vertex[0].u = 0.0;
    polygon[13].Vertex[0].v = 0.0;
    polygon[13].Vertex[1].u = 1.0;
    polygon[13].Vertex[1].v = 0.0;
    polygon[13].Vertex[2].u = 1.0;
    polygon[13].Vertex[2].v = 1.0;

    polygon[14].Scale[0] = 2.0;
    polygon[14].Scale[1] = 1.0;
    polygon[14].Shift[0] = 0.0;
    polygon[14].Shift[1] = 0.0;
    polygon[14].Rotate = 0.0;
    polygon[14].Vertex[0] = vertex[7];
    polygon[14].Vertex[1] = vertex[24];
    polygon[14].Vertex[2] = vertex[23];
    polygon[14].Texture = texture[2].TexID;
    polygon[14].Vertex[0].u = 0.0;
    polygon[14].Vertex[0].v = 0.0;
    polygon[14].Vertex[1].u = 1.0;
    polygon[14].Vertex[1].v = 1.0;
    polygon[14].Vertex[2].u = 0.0;
    polygon[14].Vertex[2].v = 1.0;

    polygon[15].Scale[0] = 2.0;
    polygon[15].Scale[1] = 1.0;
    polygon[15].Shift[0] = 0.0;
    polygon[15].Shift[1] = 0.0;
    polygon[15].Rotate = 0.0;
    polygon[15].Vertex[0] = vertex[7];
    polygon[15].Vertex[1] = vertex[8];
    polygon[15].Vertex[2] = vertex[24];
    polygon[15].Texture = texture[2].TexID;
    polygon[15].Vertex[0].u = 0.0;
    polygon[15].Vertex[0].v = 0.0;
    polygon[15].Vertex[1].u = 1.0;
    polygon[15].Vertex[1].v = 0.0;
    polygon[15].Vertex[2].u = 1.0;
    polygon[15].Vertex[2].v = 1.0;

    polygon[16].Scale[0] = 2.0;
    polygon[16].Scale[1] = 1.0;
    polygon[16].Shift[0] = 0.0;
    polygon[16].Shift[1] = 0.0;
    polygon[16].Rotate = 0.0;
    polygon[16].Vertex[0] = vertex[8];
    polygon[16].Vertex[1] = vertex[25];
    polygon[16].Vertex[2] = vertex[24];
    polygon[16].Texture = texture[2].TexID;
    polygon[16].Vertex[0].u = 0.0;
    polygon[16].Vertex[0].v = 0.0;
    polygon[16].Vertex[1].u = 1.0;
    polygon[16].Vertex[1].v = 1.0;
    polygon[16].Vertex[2].u = 0.0;
    polygon[16].Vertex[2].v = 1.0;

    polygon[17].Scale[0] = 2.0;
    polygon[17].Scale[1] = 1.0;
    polygon[17].Shift[0] = 0.0;
    polygon[17].Shift[1] = 0.0;
    polygon[17].Rotate = 0.0;
    polygon[17].Vertex[0] = vertex[8];
    polygon[17].Vertex[1] = vertex[9];
    polygon[17].Vertex[2] = vertex[25];
    polygon[17].Texture = texture[2].TexID;
    polygon[17].Vertex[0].u = 0.0;
    polygon[17].Vertex[0].v = 0.0;
    polygon[17].Vertex[1].u = 1.0;
    polygon[17].Vertex[1].v = 0.0;
    polygon[17].Vertex[2].u = 1.0;
    polygon[17].Vertex[2].v = 1.0;

    polygon[18].Scale[0] = 2.0;
    polygon[18].Scale[1] = 1.0;
    polygon[18].Shift[0] = 0.0;
    polygon[18].Shift[1] = 20.0;
    polygon[18].Rotate = 0.0;
    polygon[18].Vertex[0] = vertex[9];
    polygon[18].Vertex[1] = vertex[26];
    polygon[18].Vertex[2] = vertex[25];
    polygon[18].Texture = texture[2].TexID;
    polygon[18].Vertex[0].u = 0.0;
    polygon[18].Vertex[0].v = 0.0;
    polygon[18].Vertex[1].u = 1.0;
    polygon[18].Vertex[1].v = 1.0;
    polygon[18].Vertex[2].u = 0.0;
    polygon[18].Vertex[2].v = 1.0;

    polygon[19].Scale[0] = 2.0;
    polygon[19].Scale[1] = 1.0;
    polygon[19].Shift[0] = 0.0;
    polygon[19].Shift[1] = 20.0;
    polygon[19].Rotate = 0.0;
    polygon[19].Vertex[0] = vertex[9];
    polygon[19].Vertex[1] = vertex[10];
    polygon[19].Vertex[2] = vertex[26];
    polygon[19].Texture = texture[2].TexID;
    polygon[19].Vertex[0].u = 0.0;
    polygon[19].Vertex[0].v = 0.0;
    polygon[19].Vertex[1].u = 1.0;
    polygon[19].Vertex[1].v = 0.0;
    polygon[19].Vertex[2].u = 1.0;
    polygon[19].Vertex[2].v = 1.0;

    polygon[20].Scale[0] = 2.0;
    polygon[20].Scale[1] = 1.0;
    polygon[20].Shift[0] = 0.0;
    polygon[20].Shift[1] = 0.0;
    polygon[20].Rotate = 0.0;
    polygon[20].Vertex[0] = vertex[10];
    polygon[20].Vertex[1] = vertex[27];
    polygon[20].Vertex[2] = vertex[26];
    polygon[20].Texture = texture[2].TexID;
    polygon[20].Vertex[0].u = 0.0;
    polygon[20].Vertex[0].v = 0.0;
    polygon[20].Vertex[1].u = 1.0;
    polygon[20].Vertex[1].v = 1.0;
    polygon[20].Vertex[2].u = 0.0;
    polygon[20].Vertex[2].v = 1.0;

    polygon[21].Scale[0] = 2.0;
    polygon[21].Scale[1] = 1.0;
    polygon[21].Shift[0] = 0.0;
    polygon[21].Shift[1] = 0.0;
    polygon[21].Rotate = 0.0;
    polygon[21].Vertex[0] = vertex[10];
    polygon[21].Vertex[1] = vertex[11];
    polygon[21].Vertex[2] = vertex[27];
    polygon[21].Texture = texture[2].TexID;
    polygon[21].Vertex[0].u = 0.0;
    polygon[21].Vertex[0].v = 0.0;
    polygon[21].Vertex[1].u = 1.0;
    polygon[21].Vertex[1].v = 0.0;
    polygon[21].Vertex[2].u = 1.0;
    polygon[21].Vertex[2].v = 1.0;

    polygon[22].Scale[0] = 2.0;
    polygon[22].Scale[1] = 1.0;
    polygon[22].Shift[0] = 0.0;
    polygon[22].Shift[1] = 0.0;
    polygon[22].Rotate = 0.0;
    polygon[22].Vertex[0] = vertex[11];
    polygon[22].Vertex[1] = vertex[28];
    polygon[22].Vertex[2] = vertex[27];
    polygon[22].Texture = texture[2].TexID;
    polygon[22].Vertex[0].u = 0.0;
    polygon[22].Vertex[0].v = 0.0;
    polygon[22].Vertex[1].u = 1.0;
    polygon[22].Vertex[1].v = 1.0;
    polygon[22].Vertex[2].u = 0.0;
    polygon[22].Vertex[2].v = 1.0;

    polygon[23].Scale[0] = 2.0;
    polygon[23].Scale[1] = 1.0;
    polygon[23].Shift[0] = 0.0;
    polygon[23].Shift[1] = 0.0;
    polygon[23].Rotate = 0.0;
    polygon[23].Vertex[0] = vertex[11];
    polygon[23].Vertex[1] = vertex[12];
    polygon[23].Vertex[2] = vertex[28];
    polygon[23].Texture = texture[2].TexID;
    polygon[23].Vertex[0].u = 0.0;
    polygon[23].Vertex[0].v = 0.0;
    polygon[23].Vertex[1].u = 1.0;
    polygon[23].Vertex[1].v = 0.0;
    polygon[23].Vertex[2].u = 1.0;
    polygon[23].Vertex[2].v = 1.0;


    polygon[24].Scale[0] = 2.0;
    polygon[24].Scale[1] = 1.0;
    polygon[24].Shift[0] = 0.0;
    polygon[24].Shift[1] = 0.0;
    polygon[24].Rotate = 0.0;
    polygon[24].Vertex[0] = vertex[12];
    polygon[24].Vertex[1] = vertex[29];
    polygon[24].Vertex[2] = vertex[28];
    polygon[24].Texture = texture[2].TexID;
    polygon[24].Vertex[0].u = 0.0;
    polygon[24].Vertex[0].v = 0.0;
    polygon[24].Vertex[1].u = 1.0;
    polygon[24].Vertex[1].v = 1.0;
    polygon[24].Vertex[2].u = 0.0;
    polygon[24].Vertex[2].v = 1.0;

    polygon[25].Scale[0] = 2.0;
    polygon[25].Scale[1] = 1.0;
    polygon[25].Shift[0] = 0.0;
    polygon[25].Shift[1] = 0.0;
    polygon[25].Rotate = 0.0;
    polygon[25].Vertex[0] = vertex[12];
    polygon[25].Vertex[1] = vertex[13];
    polygon[25].Vertex[2] = vertex[29];
    polygon[25].Texture = texture[2].TexID;
    polygon[25].Vertex[0].u = 0.0;
    polygon[25].Vertex[0].v = 0.0;
    polygon[25].Vertex[1].u = 1.0;
    polygon[25].Vertex[1].v = 0.0;
    polygon[25].Vertex[2].u = 1.0;
    polygon[25].Vertex[2].v = 1.0;

    polygon[26].Scale[0] = 1.0;
    polygon[26].Scale[1] = 1.0;
    polygon[26].Shift[0] = 0.0;
    polygon[26].Shift[1] = 0.0;
    polygon[26].Rotate = 0.0;
    polygon[26].Vertex[0] = vertex[13];
    polygon[26].Vertex[1] = vertex[30];
    polygon[26].Vertex[2] = vertex[29];
    polygon[26].Texture = texture[2].TexID;
    polygon[26].Vertex[0].u = 0.0;
    polygon[26].Vertex[0].v = 0.0;
    polygon[26].Vertex[1].u = 1.0;
    polygon[26].Vertex[1].v = 1.0;
    polygon[26].Vertex[2].u = 0.0;
    polygon[26].Vertex[2].v = 1.0;

    polygon[27].Scale[0] = 1.0;
    polygon[27].Scale[1] = 1.0;
    polygon[27].Shift[0] = 0.0;
    polygon[27].Shift[1] = 0.0;
    polygon[27].Rotate = 0.0;
    polygon[27].Vertex[0] = vertex[13];
    polygon[27].Vertex[1] = vertex[14];
    polygon[27].Vertex[2] = vertex[30];
    polygon[27].Texture = texture[2].TexID;
    polygon[27].Vertex[0].u = 0.0;
    polygon[27].Vertex[0].v = 0.0;
    polygon[27].Vertex[1].u = 1.0;
    polygon[27].Vertex[1].v = 0.0;
    polygon[27].Vertex[2].u = 1.0;
    polygon[27].Vertex[2].v = 1.0;

    polygon[28].Scale[0] = 6.0;
    polygon[28].Scale[1] = 1.0;
    polygon[28].Shift[0] = 0.0;
    polygon[28].Shift[1] = 0.0;
    polygon[28].Rotate = 0.0;
    polygon[28].Vertex[0] = vertex[14];
    polygon[28].Vertex[1] = vertex[31];
    polygon[28].Vertex[2] = vertex[30];
    polygon[28].Texture = texture[2].TexID;
    polygon[28].Vertex[0].u = 0.0;
    polygon[28].Vertex[0].v = 0.0;
    polygon[28].Vertex[1].u = 1.0;
    polygon[28].Vertex[1].v = 1.0;
    polygon[28].Vertex[2].u = 0.0;
    polygon[28].Vertex[2].v = 1.0;

    polygon[29].Scale[0] = 6.0;
    polygon[29].Scale[1] = 1.0;
    polygon[29].Shift[0] = 0.0;
    polygon[29].Shift[1] = 0.0;
    polygon[29].Rotate = 0.0;
    polygon[29].Vertex[0] = vertex[14];
    polygon[29].Vertex[1] = vertex[15];
    polygon[29].Vertex[2] = vertex[31];
    polygon[29].Texture = texture[2].TexID;
    polygon[29].Vertex[0].u = 0.0;
    polygon[29].Vertex[0].v = 0.0;
    polygon[29].Vertex[1].u = 1.0;
    polygon[29].Vertex[1].v = 0.0;
    polygon[29].Vertex[2].u = 1.0;
    polygon[29].Vertex[2].v = 1.0;

    polygon[30].Scale[0] = 5.0;
    polygon[30].Scale[1] = 1.0;
    polygon[30].Shift[0] = 0.0;
    polygon[30].Shift[1] = 0.0;
    polygon[30].Rotate = 0.0;
    polygon[30].Vertex[0] = vertex[15];
    polygon[30].Vertex[1] = vertex[16];
    polygon[30].Vertex[2] = vertex[31];
    polygon[30].Texture = texture[2].TexID;
    polygon[30].Vertex[0].u = 0.0;
    polygon[30].Vertex[0].v = 0.0;
    polygon[30].Vertex[1].u = 1.0;
    polygon[30].Vertex[1].v = 1.0;
    polygon[30].Vertex[2].u = 0.0;
    polygon[30].Vertex[2].v = 1.0;

    polygon[31].Scale[0] = 5.0;
    polygon[31].Scale[1] = 1.0;
    polygon[31].Shift[0] = 0.0;
    polygon[31].Shift[1] = 0.0;
    polygon[31].Rotate = 0.0;
    polygon[31].Vertex[0] = vertex[15];
    polygon[31].Vertex[1] = vertex[0];
    polygon[31].Vertex[2] = vertex[16];
    polygon[31].Texture = texture[2].TexID;
    polygon[31].Vertex[0].u = 0.0;
    polygon[31].Vertex[0].v = 0.0;
    polygon[31].Vertex[1].u = 1.0;
    polygon[31].Vertex[1].v = 0.0;
    polygon[31].Vertex[2].u = 1.0;
    polygon[31].Vertex[2].v = 1.0;

    polygon[32].Scale[0] = 5.0;
    polygon[32].Scale[1] = 5.0;
    polygon[32].Shift[0] = 0.0;
    polygon[32].Shift[1] = 0.0;
    polygon[32].Rotate = 0.0;
    polygon[32].Vertex[0] = vertex[15];
    polygon[32].Vertex[1] = vertex[1];
    polygon[32].Vertex[2] = vertex[0];
    polygon[32].Texture = texture[0].TexID;
    polygon[32].Vertex[0].u = 0.0;
    polygon[32].Vertex[0].v = 0.0;
    polygon[32].Vertex[1].u = 1.0;
    polygon[32].Vertex[1].v = 1.0;
    polygon[32].Vertex[2].u = 0.0;
    polygon[32].Vertex[2].v = 1.0;

    polygon[33].Scale[0] = 5.0;
    polygon[33].Scale[1] = 5.0;
    polygon[33].Shift[0] = 0.0;
    polygon[33].Shift[1] = 0.0;
    polygon[33].Rotate = 0.0;
    polygon[33].Vertex[0] = vertex[15];
    polygon[33].Vertex[1] = vertex[14];
    polygon[33].Vertex[2] = vertex[1];
    polygon[33].Texture = texture[0].TexID;
    polygon[33].Vertex[0].u = 0.0;
    polygon[33].Vertex[0].v = 0.0;
    polygon[33].Vertex[1].u = 1.0;
    polygon[33].Vertex[1].v = 0.0;
    polygon[33].Vertex[2].u = 1.0;
    polygon[33].Vertex[2].v = 1.0;

    polygon[34].Scale[0] = 5.0;
    polygon[34].Scale[1] = 5.0;
    polygon[34].Shift[0] = 0.0;
    polygon[34].Shift[1] = 0.0;
    polygon[34].Rotate = 0.0;
    polygon[34].Vertex[0] = vertex[13];
    polygon[34].Vertex[1] = vertex[3];
    polygon[34].Vertex[2] = vertex[2];
    polygon[34].Texture = texture[0].TexID;
    polygon[34].Vertex[0].u = 0.0;
    polygon[34].Vertex[0].v = 0.0;
    polygon[34].Vertex[1].u = 1.0;
    polygon[34].Vertex[1].v = 1.0;
    polygon[34].Vertex[2].u = 0.0;
    polygon[34].Vertex[2].v = 1.0;

    polygon[35].Scale[0] = 5.0;
    polygon[35].Scale[1] = 5.0;
    polygon[35].Shift[0] = 0.0;
    polygon[35].Shift[1] = 0.0;
    polygon[35].Rotate = 0.0;
    polygon[35].Vertex[0] = vertex[13];
    polygon[35].Vertex[1] = vertex[4];
    polygon[35].Vertex[2] = vertex[3];
    polygon[35].Texture = texture[0].TexID;
    polygon[35].Vertex[0].u = 0.0;
    polygon[35].Vertex[0].v = 0.0;
    polygon[35].Vertex[1].u = 1.0;
    polygon[35].Vertex[1].v = 0.0;
    polygon[35].Vertex[2].u = 1.0;
    polygon[35].Vertex[2].v = 1.0;

    polygon[36].Scale[0] = 1.0;
    polygon[36].Scale[1] = 2.0;
    polygon[36].Shift[0] = 0.0;
    polygon[36].Shift[1] = 0.0;
    polygon[36].Rotate = 0.0;
    polygon[36].Vertex[0] = vertex[11];
    polygon[36].Vertex[1] = vertex[5];
    polygon[36].Vertex[2] = vertex[12];
    polygon[36].Texture = texture[0].TexID;
    polygon[36].Vertex[0].u = 0.0;
    polygon[36].Vertex[0].v = 0.0;
    polygon[36].Vertex[1].u = 1.0;
    polygon[36].Vertex[1].v = 1.0;
    polygon[36].Vertex[2].u = 0.0;
    polygon[36].Vertex[2].v = 1.0;

    polygon[37].Scale[0] = 1.0;
    polygon[37].Scale[1] = 2.0;
    polygon[37].Shift[0] = 0.0;
    polygon[37].Shift[1] = 0.0;
    polygon[37].Rotate = 0.0;
    polygon[37].Vertex[0] = vertex[11];
    polygon[37].Vertex[1] = vertex[6];
    polygon[37].Vertex[2] = vertex[5];
    polygon[37].Texture = texture[0].TexID;
    polygon[37].Vertex[0].u = 0.0;
    polygon[37].Vertex[0].v = 0.0;
    polygon[37].Vertex[1].u = 1.0;
    polygon[37].Vertex[1].v = 0.0;
    polygon[37].Vertex[2].u = 1.0;
    polygon[37].Vertex[2].v = 1.0;

    polygon[38].Scale[0] = 5.0;
    polygon[38].Scale[1] = 5.0;
    polygon[38].Shift[0] = 0.0;
    polygon[38].Shift[1] = 0.0;
    polygon[38].Rotate = 0.0;
    polygon[38].Vertex[0] = vertex[9];
    polygon[38].Vertex[1] = vertex[7];
    polygon[38].Vertex[2] = vertex[10];
    polygon[38].Texture = texture[0].TexID;
    polygon[38].Vertex[0].u = 0.0;
    polygon[38].Vertex[0].v = 0.0;
    polygon[38].Vertex[1].u = 1.0;
    polygon[38].Vertex[1].v = 1.0;
    polygon[38].Vertex[2].u = 0.0;
    polygon[38].Vertex[2].v = 1.0;

    polygon[39].Scale[0] = 5.0;
    polygon[39].Scale[1] = 5.0;
    polygon[39].Shift[0] = 0.0;
    polygon[39].Shift[1] = 0.0;
    polygon[39].Rotate = 0.0;
    polygon[39].Vertex[0] = vertex[9];
    polygon[39].Vertex[1] = vertex[8];
    polygon[39].Vertex[2] = vertex[7];
    polygon[39].Texture = texture[0].TexID;
    polygon[39].Vertex[0].u = 0.0;
    polygon[39].Vertex[0].v = 0.0;
    polygon[39].Vertex[1].u = 1.0;
    polygon[39].Vertex[1].v = 0.0;
    polygon[39].Vertex[2].u = 1.0;
    polygon[39].Vertex[2].v = 1.0;

    polygon[40].Scale[0] = 5.0;
    polygon[40].Scale[1] = 1.0;
    polygon[40].Shift[0] = 0.0;
    polygon[40].Shift[1] = 0.0;
    polygon[40].Rotate = 0.0;
    polygon[40].Vertex[0] = vertex[18];
    polygon[40].Vertex[1] = vertex[16];
    polygon[40].Vertex[2] = vertex[17];
    polygon[40].Texture = texture[1].TexID;
    polygon[40].Vertex[0].u = 0.0;
    polygon[40].Vertex[0].v = 0.0;
    polygon[40].Vertex[1].u = 1.0;
    polygon[40].Vertex[1].v = 1.0;
    polygon[40].Vertex[2].u = 0.0;
    polygon[40].Vertex[2].v = 1.0;

    polygon[41].Scale[0] = 5.0;
    polygon[41].Scale[1] = 1.0;
    polygon[41].Shift[0] = 0.0;
    polygon[41].Shift[1] = 0.0;
    polygon[41].Rotate = 0.0;
    polygon[41].Vertex[0] = vertex[18];
    polygon[41].Vertex[1] = vertex[34];
    polygon[41].Vertex[2] = vertex[16];
    polygon[41].Texture = texture[1].TexID;
    polygon[41].Vertex[0].u = 0.0;
    polygon[41].Vertex[0].v = 0.0;
    polygon[41].Vertex[1].u = 1.0;
    polygon[41].Vertex[1].v = 0.0;
    polygon[41].Vertex[2].u = 1.0;
    polygon[41].Vertex[2].v = 1.0;

    polygon[42].Scale[0] = 3.0;
    polygon[42].Scale[1] = 2.0;
    polygon[42].Shift[0] = 0.0;
    polygon[42].Shift[1] = 0.0;
    polygon[42].Rotate = 0.0;
    polygon[42].Vertex[0] = vertex[36];
    polygon[42].Vertex[1] = vertex[18];
    polygon[42].Vertex[2] = vertex[19];
    polygon[42].Texture = texture[3].TexID;
    polygon[42].Vertex[0].u = 0.0;
    polygon[42].Vertex[0].v = 0.0;
    polygon[42].Vertex[1].u = 1.0;
    polygon[42].Vertex[1].v = 1.0;
    polygon[42].Vertex[2].u = 0.0;
    polygon[42].Vertex[2].v = 1.0;

    polygon[43].Scale[0] = 3.0;
    polygon[43].Scale[1] = 2.0;
    polygon[43].Shift[0] = 0.0;
    polygon[43].Shift[1] = 0.0;
    polygon[43].Rotate = 0.0;
    polygon[43].Vertex[0] = vertex[36];
    polygon[43].Vertex[1] = vertex[35];
    polygon[43].Vertex[2] = vertex[18];
    polygon[43].Texture = texture[3].TexID;
    polygon[43].Vertex[0].u = 0.0;
    polygon[43].Vertex[0].v = 0.0;
    polygon[43].Vertex[1].u = 1.0;
    polygon[43].Vertex[1].v = 0.0;
    polygon[43].Vertex[2].u = 1.0;
    polygon[43].Vertex[2].v = 1.0;

    polygon[44].Scale[0] = 1.0;
    polygon[44].Scale[1] = 2.0;
    polygon[44].Shift[0] = 0.0;
    polygon[44].Shift[1] = 0.0;
    polygon[44].Rotate = 0.0;
    polygon[44].Vertex[0] = vertex[22];
    polygon[44].Vertex[1] = vertex[28];
    polygon[44].Vertex[2] = vertex[21];
    polygon[44].Texture = texture[1].TexID;
    polygon[44].Vertex[0].u = 0.0;
    polygon[44].Vertex[0].v = 0.0;
    polygon[44].Vertex[1].u = 1.0;
    polygon[44].Vertex[1].v = 1.0;
    polygon[44].Vertex[2].u = 0.0;
    polygon[44].Vertex[2].v = 1.0;

    polygon[45].Scale[0] = 1.0;
    polygon[45].Scale[1] = 2.0;
    polygon[45].Shift[0] = 0.0;
    polygon[45].Shift[1] = 0.0;
    polygon[45].Rotate = 0.0;
    polygon[45].Vertex[0] = vertex[22];
    polygon[45].Vertex[1] = vertex[27];
    polygon[45].Vertex[2] = vertex[28];
    polygon[45].Texture = texture[1].TexID;
    polygon[45].Vertex[0].u = 0.0;
    polygon[45].Vertex[0].v = 0.0;
    polygon[45].Vertex[1].u = 1.0;
    polygon[45].Vertex[1].v = 0.0;
    polygon[45].Vertex[2].u = 1.0;
    polygon[45].Vertex[2].v = 1.0;

    polygon[46].Scale[0] = 5.0;
    polygon[46].Scale[1] = 5.0;
    polygon[46].Shift[0] = 0.0;
    polygon[46].Shift[1] = 0.0;
    polygon[46].Rotate = 0.0;
    polygon[46].Vertex[0] = vertex[24];
    polygon[46].Vertex[1] = vertex[26];
    polygon[46].Vertex[2] = vertex[23];
    polygon[46].Texture = texture[1].TexID;
    polygon[46].Vertex[0].u = 0.0;
    polygon[46].Vertex[0].v = 0.0;
    polygon[46].Vertex[1].u = 1.0;
    polygon[46].Vertex[1].v = 1.0;
    polygon[46].Vertex[2].u = 0.0;
    polygon[46].Vertex[2].v = 1.0;

    polygon[47].Scale[0] = 5.0;
    polygon[47].Scale[1] = 5.0;
    polygon[47].Shift[0] = 0.0;
    polygon[47].Shift[1] = 0.0;
    polygon[47].Rotate = 0.0;
    polygon[47].Vertex[0] = vertex[24];
    polygon[47].Vertex[1] = vertex[25];
    polygon[47].Vertex[2] = vertex[26];
    polygon[47].Texture = texture[1].TexID;
    polygon[47].Vertex[0].u = 0.0;
    polygon[47].Vertex[0].v = 0.0;
    polygon[47].Vertex[1].u = 1.0;
    polygon[47].Vertex[1].v = 0.0;
    polygon[47].Vertex[2].u = 1.0;
    polygon[47].Vertex[2].v = 1.0;

    polygon[48].Scale[0] = 3.0;
    polygon[48].Scale[1] = 2.0;
    polygon[48].Shift[0] = 0.0;
    polygon[48].Shift[1] = 0.0;
    polygon[48].Rotate = 0.0;
    polygon[48].Vertex[0] = vertex[20];
    polygon[48].Vertex[1] = vertex[35];
    polygon[48].Vertex[2] = vertex[36];
    polygon[48].Texture = texture[3].TexID;
    polygon[48].Vertex[0].u = 0.0;
    polygon[48].Vertex[0].v = 0.0;
    polygon[48].Vertex[1].u = 1.0;
    polygon[48].Vertex[1].v = 1.0;
    polygon[48].Vertex[2].u = 0.0;
    polygon[48].Vertex[2].v = 1.0;

    polygon[49].Scale[0] = 3.0;
    polygon[49].Scale[1] = 2.0;
    polygon[49].Shift[0] = 0.0;
    polygon[49].Shift[1] = 0.0;
    polygon[49].Rotate = 0.0;
    polygon[49].Vertex[0] = vertex[20];
    polygon[49].Vertex[1] = vertex[29];
    polygon[49].Vertex[2] = vertex[35];
    polygon[49].Texture = texture[3].TexID;
    polygon[49].Vertex[0].u = 0.0;
    polygon[49].Vertex[0].v = 0.0;
    polygon[49].Vertex[1].u = 1.0;
    polygon[49].Vertex[1].v = 0.0;
    polygon[49].Vertex[2].u = 1.0;
    polygon[49].Vertex[2].v = 1.0;

    polygon[50].Scale[0] = 5.0;
    polygon[50].Scale[1] = 5.0;
    polygon[50].Shift[0] = 0.0;
    polygon[50].Shift[1] = 0.0;
    polygon[50].Rotate = 0.0;
    polygon[50].Vertex[0] = vertex[30];
    polygon[50].Vertex[1] = vertex[32];
    polygon[50].Vertex[2] = vertex[29];
    polygon[50].Texture = texture[1].TexID;
    polygon[50].Vertex[0].u = 0.0;
    polygon[50].Vertex[0].v = 0.0;
    polygon[50].Vertex[1].u = 1.0;
    polygon[50].Vertex[1].v = 1.0;
    polygon[50].Vertex[2].u = 0.0;
    polygon[50].Vertex[2].v = 1.0;

    polygon[51].Scale[0] = 5.0;
    polygon[51].Scale[1] = 5.0;
    polygon[51].Shift[0] = 0.0;
    polygon[51].Shift[1] = 0.0;
    polygon[51].Rotate = 0.0;
    polygon[51].Vertex[0] = vertex[30];
    polygon[51].Vertex[1] = vertex[31];
    polygon[51].Vertex[2] = vertex[32];
    polygon[51].Texture = texture[1].TexID;
    polygon[51].Vertex[0].u = 0.0;
    polygon[51].Vertex[0].v = 0.0;
    polygon[51].Vertex[1].u = 1.0;
    polygon[51].Vertex[1].v = 0.0;
    polygon[51].Vertex[2].u = 1.0;
    polygon[51].Vertex[2].v = 1.0;

    polygon[52].Scale[0] = 3.0;
    polygon[52].Scale[1] = 2.0;
    polygon[52].Shift[0] = 0.0;
    polygon[52].Shift[1] = 0.0;
    polygon[52].Rotate = 0.0;
    polygon[52].Vertex[0] = vertex[29];
    polygon[52].Vertex[1] = vertex[33];
    polygon[52].Vertex[2] = vertex[35];
    polygon[52].Texture = texture[3].TexID;
    polygon[52].Vertex[0].u = 0.0;
    polygon[52].Vertex[0].v = 0.0;
    polygon[52].Vertex[1].u = 1.0;
    polygon[52].Vertex[1].v = 1.0;
    polygon[52].Vertex[2].u = 0.0;
    polygon[52].Vertex[2].v = 1.0;

    polygon[53].Scale[0] = 3.0;
    polygon[53].Scale[1] = 2.0;
    polygon[53].Shift[0] = 0.0;
    polygon[53].Shift[1] = 0.0;
    polygon[53].Rotate = 0.0;
    polygon[53].Vertex[0] = vertex[29];
    polygon[53].Vertex[1] = vertex[32];
    polygon[53].Vertex[2] = vertex[33];
    polygon[53].Texture = texture[3].TexID;
    polygon[53].Vertex[0].u = 0.0;
    polygon[53].Vertex[0].v = 0.0;
    polygon[53].Vertex[1].u = 1.0;
    polygon[53].Vertex[1].v = 0.0;
    polygon[53].Vertex[2].u = 1.0;
    polygon[53].Vertex[2].v = 1.0;

    polygon[54].Scale[0] = 3.0;
    polygon[54].Scale[1] = 2.0;
    polygon[54].Shift[0] = 0.0;
    polygon[54].Shift[1] = 0.0;
    polygon[54].Rotate = 0.0;
    polygon[54].Vertex[0] = vertex[35];
    polygon[54].Vertex[1] = vertex[34];
    polygon[54].Vertex[2] = vertex[18];
    polygon[54].Texture = texture[3].TexID;
    polygon[54].Vertex[0].u = 0.0;
    polygon[54].Vertex[0].v = 0.0;
    polygon[54].Vertex[1].u = 1.0;
    polygon[54].Vertex[1].v = 1.0;
    polygon[54].Vertex[2].u = 0.0;
    polygon[54].Vertex[2].v = 1.0;

    polygon[55].Scale[0] = 3.0;
    polygon[55].Scale[1] = 2.0;
    polygon[55].Shift[0] = 0.0;
    polygon[55].Shift[1] = 0.0;
    polygon[55].Rotate = 0.0;
    polygon[55].Vertex[0] = vertex[35];
    polygon[55].Vertex[1] = vertex[33];
    polygon[55].Vertex[2] = vertex[34];
    polygon[55].Texture = texture[3].TexID;
    polygon[55].Vertex[0].u = 0.0;
    polygon[55].Vertex[0].v = 0.0;
    polygon[55].Vertex[1].u = 1.0;
    polygon[55].Vertex[1].v = 0.0;
    polygon[55].Vertex[2].u = 1.0;
    polygon[55].Vertex[2].v = 1.0;

    polygon[56].Scale[0] = 4.0;
    polygon[56].Scale[1] = 3.0;
    polygon[56].Shift[0] = 0.0;
    polygon[56].Shift[1] = 0.0;
    polygon[56].Rotate = 0.0;
    polygon[56].Vertex[0] = vertex[32];
    polygon[56].Vertex[1] = vertex[34];
    polygon[56].Vertex[2] = vertex[33];
    polygon[56].Texture = texture[2].TexID;
    polygon[56].Vertex[0].u = 0.0;
    polygon[56].Vertex[0].v = 0.0;
    polygon[56].Vertex[1].u = 1.0;
    polygon[56].Vertex[1].v = 0.0;
    polygon[56].Vertex[2].u = 0.5;
    polygon[56].Vertex[2].v = 0.5;

    polygon[57].Scale[0] = 4.0;
    polygon[57].Scale[1] = 3.0;
    polygon[57].Shift[0] = 0.0;
    polygon[57].Shift[1] = 0.0;
    polygon[57].Rotate = 0;
    polygon[57].Vertex[0] = vertex[19];
    polygon[57].Vertex[1] = vertex[20];
    polygon[57].Vertex[2] = vertex[36];
    polygon[57].Texture = texture[2].TexID;
    polygon[57].Vertex[0].u = 0.0;
    polygon[57].Vertex[0].v = 0.0;
    polygon[57].Vertex[1].u = 1.0;
    polygon[57].Vertex[1].v = 0.0;
    polygon[57].Vertex[2].u = 0.5;
    polygon[57].Vertex[2].v = 0.5;

    for (int loop = 0; loop < numPolygons; loop++)
        polygon[loop].SetNormal();
}

void SetGLProperties()
{
    glCullFace(GL_BACK);
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClearDepth(1.0);
    glDepthFunc(GL_LESS);
    glEnable(GL_DEPTH_TEST);
    glShadeModel(GL_SMOOTH);
    glEnable(GL_NORMALIZE);
    glEnable(GL_CULL_FACE);
}

void SetGLProjection(int Width, int Height)
{
    if (Height==0)
        Height=1;
    glViewport(0, 0, Width, Height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0,(float)Width/(float)Height, 0.8, 500.0);
}

void SetGLView(int Width, int Height)
{
    SetGLProjection(Width, Height);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void SetGLMaterial()
{
    float mat_ambient[] = { 1.0, 1.0, 1.0, 1.0 };
    float mat_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
    float mat_specular[] = { 0.9, 0.9, 0.9, 1.0 };
    float mat_emission[] = { 0.0, 0.0, 0.0, 1.0 };
    float mat_shininess[] = { 80.0 };

    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
    glMaterialfv(GL_FRONT, GL_EMISSION, mat_emission);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
}

int SetGLTexture(TEXTURE* texture)
{
    glEnable(GL_TEXTURE_2D);
    
    sprintf(texture[0].TexName, "%s", "floor.tga");
    if (!texture[0].LoadTGA())
    {
        MessageBox(NULL,"Failed to load floor image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[1].TexName, "%s", "roof.tga");
    if (!texture[1].LoadTGA())
    {
        MessageBox(NULL,"Failed to load roof image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[2].TexName, "%s", "wall.tga");
    if (!texture[2].LoadTGA())
    {
        MessageBox(NULL,"Failed to load wall image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[3].TexName, "%s", "tile.tga");
    if (!texture[3].LoadTGA())
    {
        MessageBox(NULL,"Failed to load tile image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[4].TexName, "%s", "burndecal.tga");
    if (!texture[4].LoadTGA(GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR, GL_CLAMP, GL_CLAMP, 1))
    {
        MessageBox(NULL,"Failed to load burn decal image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[5].TexName, "%s", "muzzleflash.tga");
    if (!texture[5].LoadTGA(GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR, GL_CLAMP, GL_CLAMP, 1))
    {
        MessageBox(NULL,"Failed to load muzzle flash image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[6].TexName, "%s", "impactflash.tga");
    if (!texture[6].LoadTGA(GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR, GL_CLAMP, GL_CLAMP, 1))
    {
        MessageBox(NULL,"Failed to load impact flash image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[7].TexName, "%s", "bulletdecal.tga");
    if (!texture[7].LoadTGA(GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR, GL_CLAMP, GL_CLAMP, 1))
    {
        MessageBox(NULL,"Failed to load bullet decal image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[8].TexName, "%s", "particle1.tga");
    if (!texture[8].LoadTGA(GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR, GL_CLAMP, GL_CLAMP, 1))
    {
        MessageBox(NULL,"Failed to load particle1 image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    sprintf(texture[9].TexName, "%s", "particle2.tga");
    if (!texture[9].LoadTGA(GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR, GL_CLAMP, GL_CLAMP, 1))
    {
        MessageBox(NULL,"Failed to load particle2 image","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }
    return TRUE;
}

void DrawMyText()
{
    glBlendFunc(GL_ONE,  GL_ONE_MINUS_SRC_ALPHA) ;
    glDisable(GL_LIGHTING);
    glColor3f(0.0, 1.0, 0.0);
    glEnable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
    glPushMatrix();
    glLoadIdentity();
    glFontBegin(&myFont);
    char text[256];

    glFontTextOut("Tutorial #17  (Particles)", -52, 40, -100);

    sprintf(text, "FPS = %d", fps);
    glFontTextOut(text, -52, 38, -100);

    sprintf(text, "%s", "Press M to change camera mode");
    glFontTextOut(text, -52, 34, -100);

    sprintf(text, "%s", "Press F to fire");
    glFontTextOut(text, -52, 30, -100);

    glFontEnd();
    glPopMatrix();
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_BLEND);
    glEnable(GL_LIGHTING);
}

void DrawGrid()
{
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glPushMatrix();
    glTranslatef(0,-2.0,0);
    glColor3f(0.0f,1.0f,0.0f);

    float Line = -10;
    int Grid;
    glBegin(GL_LINES);
    for(Grid = 0; Grid <= 20; Grid += 1)
    {
        glVertex3f(Line + Grid, 0, -10);
        glVertex3f(Line + Grid, 0, 10);
        glVertex3f(-10, 0, Line + Grid);
        glVertex3f(10, 0, Line + Grid);
    }
    glEnd();
    glPopMatrix();
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_LIGHTING);
}

void DrawWorld(BSP_node* root)
{
    float mat_ambient[] = { 0.8, 0.8, 0.8, 1.0 };
    float mat_diffuse[] = { 0.8, 0.8, 0.8, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

    glPushMatrix();

    glDisable (GL_DEPTH_TEST);
    RenderBSP(root);
    glEnable (GL_DEPTH_TEST);

    glPopMatrix();

/*
    //Draw polygons that are in the leaf
    for (int loop = 0; loop < numPolygons; loop++)
    {
        glMatrixMode(GL_TEXTURE);
        glPushMatrix();


        glScalef(polygon[loop].Scale[0], polygon[loop].Scale[1], 1.0f);
        glTranslatef(polygon[loop].Shift[0], polygon[loop].Shift[1], 0.0f);
        glRotatef(polygon[loop].Rotate, 0.0f, 0.0f, 1.0f);

        glBindTexture(GL_TEXTURE_2D, polygon[loop].Texture);
        glBegin(GL_TRIANGLES);
        glNormal3fv(&polygon[loop].Vertex[0].nx);
        glTexCoord2f(polygon[loop].Vertex[0].u, polygon[loop].Vertex[0].v);
        glVertex3fv(&polygon[loop].Vertex[0].x);
        glTexCoord2f(polygon[loop].Vertex[1].u, polygon[loop].Vertex[1].v);
        glVertex3fv(&polygon[loop].Vertex[1].x);
        glTexCoord2f(polygon[loop].Vertex[2].u, polygon[loop].Vertex[2].v);
        glVertex3fv(&polygon[loop].Vertex[2].x);
        glEnd();

        glPopMatrix();
        glMatrixMode(GL_MODELVIEW);
    }
*/
}

void DrawGreenSphere()
{
    float mat_ambient[] = { 0.2, 1.0, 0.1, 1.0 };
    float mat_diffuse[] = { 0.2, 1.0, 0.1, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
//    glDisable(GL_TEXTURE_2D);
    glPushMatrix();
    glLoadIdentity();
    glTranslatef(-2.0, -2.0, -8.0);
    GLUquadricObj * sphere = gluNewQuadric();
    gluQuadricOrientation(sphere, GLU_OUTSIDE);
    gluSphere(sphere,0.3,20,20);
    glPopMatrix();
//    glEnable(GL_TEXTURE_2D);
}

void DrawSphere()
{
    float mat_ambient[] = { 0.8, 0.5, 0.1, 1.0 };
    float mat_diffuse[] = { 0.8, 0.5, 0.1, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

    glDisable(GL_TEXTURE_2D);
    glPushMatrix();
    glTranslatef(-3.0f,-1.0f,-8.0f);
    GLUquadricObj * sphere = gluNewQuadric();
    gluQuadricOrientation(sphere, GLU_OUTSIDE);
    gluSphere(sphere, 1.0, 50, 50);
    glPopMatrix();
    glEnable(GL_TEXTURE_2D);
}

void DrawCone()
{
    float mat_ambient[] = { 0.1, 0.5, 1.0, 1.0 };
    float mat_diffuse[] = { 0.1, 0.5, 1.0, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);

    glDisable(GL_TEXTURE_2D);
    glDisable(GL_CULL_FACE);
    glPushMatrix();
    glTranslatef(3.0f,-2.0f,-8.0f);
    glRotatef(-90,1,0,0);
    GLUquadricObj * cylinder = gluNewQuadric();
    gluQuadricOrientation(cylinder, GLU_OUTSIDE);
    gluCylinder(cylinder,1.0,0.0,2.0,20,20);
    glPopMatrix();
    glEnable(GL_CULL_FACE);
    glEnable(GL_TEXTURE_2D);
}

float GetTimePassed(float& lasttime, int average, float* lastmultiplier)
{
    float timeoffset;
    float currenttime;
/*
If the program has just started, set last tickcount to the current tickcount.
This prevents the program from thinking that several hours have passed since the last frame.
*/
    if (lasttime == 0)
        lasttime = (float)GetTickCount();
        // Get the current time
    currenttime = (float)GetTickCount();
        // Calculate the offset
    timeoffset = currenttime - lasttime;
    // If timeoffset less than 1/120 of a second (in milliseconds) then set to minimum offset
    if (timeoffset < 8.333333)
    {
        timeoffset = 8.333333;
        currenttime = lasttime + timeoffset;
    }
    // Put the current time in the lasttime variable
        lasttime = currenttime;
    // return the time offset in seconds per frame
        multiplier = timeoffset / 1000;
    for (int loop = 0; loop < average - 1; loop++)
    {
        lastmultiplier[loop] = lastmultiplier[loop + 1];
    }
    lastmultiplier[average - 1] = multiplier;
    for (int loop = 0; loop < average - 1; loop++)
        multiplier += lastmultiplier[loop];
    multiplier /= (float)average; 
        if (multiplier)
        fps = (int)(1.0 / multiplier);
    return multiplier;
}

bool CheckClipPlanes(CAMERA Camera, VECTOR Vect)
{
  float ProjectionMatrix[16];
  float ModelViewMatrix[16];
  float A, B, C, Distance;
  int Counter = 0;
  glGetFloatv(GL_PROJECTION_MATRIX, ProjectionMatrix);
  glGetFloatv(GL_MODELVIEW_MATRIX, ModelViewMatrix);

  MultMatrix(ProjectionMatrix, ModelViewMatrix);

  //right clipping plane
  A = ProjectionMatrix[0] - ProjectionMatrix[3];
  B = ProjectionMatrix[4] - ProjectionMatrix[7];
  C = ProjectionMatrix[8] - ProjectionMatrix[11];

  Distance = -1 * (A * (-Camera.Position.x + Vect.x) + B * (-Camera.Position.y + Vect.y) + C * (-Camera.Position.z + Vect.z));
  if (Distance > 0)
    Counter++;

  //left clipping plane
  A = ProjectionMatrix[0] + ProjectionMatrix[3];
  B = ProjectionMatrix[4] + ProjectionMatrix[7];
  C = ProjectionMatrix[8] + ProjectionMatrix[11];

  Distance = A * (-Camera.Position.x + Vect.x) + B * (-Camera.Position.y + Vect.y) + C * (-Camera.Position.z + Vect.z);
  if (Distance > 0)
    Counter++;

  //top clipping plane
  A = ProjectionMatrix[1] - ProjectionMatrix[3];
  B = ProjectionMatrix[5] - ProjectionMatrix[7];
  C = ProjectionMatrix[9] - ProjectionMatrix[11];

  Distance = -1 * (A * (-Camera.Position.x + Vect.x) + B * (-Camera.Position.y + Vect.y) + C * (-Camera.Position.z + Vect.z));
  if (Distance > 0)
    Counter++;

  //bottom clipping plane
  A = ProjectionMatrix[1] + ProjectionMatrix[3];
  B = ProjectionMatrix[5] + ProjectionMatrix[7];
  C = ProjectionMatrix[9] + ProjectionMatrix[11];

  Distance = A * (-Camera.Position.x + Vect.x) + B * (-Camera.Position.y + Vect.y) + C * (-Camera.Position.z + Vect.z);
  if (Distance > 0)
    Counter++;

  // near clipping plane (might not be necessary when the near frustrum plane is close, but just in case)
  A = ProjectionMatrix[2] - ProjectionMatrix[3];
  B = ProjectionMatrix[6] - ProjectionMatrix[7];
  C = ProjectionMatrix[10] - ProjectionMatrix[11];

  Distance = A * (-Camera.Position.x + Vect.x) + B * (-Camera.Position.y + Vect.y) + C * (-Camera.Position.z + Vect.z);
  if (Distance > 0)
    Counter++;

  // far clipping plane (the equation didn't work for the far plane, so I'll just use a distance test)
  VECTOR Vect2;
  Vect2.x = Vect.x - Camera.Position.x;
  Vect2.y = Vect.y - Camera.Position.y;
  Vect2.z = Vect.z - Camera.Position.z;
  if (MagnitudeVector(Vect2) < 2000000)
    Counter++;

  if (Counter == 6)
    return 1;
  else
    return 0;
}

//  A halo type billboard that always faces the camera
void DrawHalo(TEXTURE* texture, LIGHT* light, CAMERA* camera)
{
    float halfwidth = 2;
    float halfheight = 2;

    MATRIX mat;
    glGetFloatv(GL_MODELVIEW_MATRIX, mat.Element);

    VECTOR right;
    right.x = mat.Element[0];
    right.y = mat.Element[4];
    right.z = mat.Element[8];
    right.Normalize();
    right.x *= halfwidth;
    right.y *= halfwidth;
    right.z *= halfwidth;

    VECTOR up;
    up.x = mat.Element[1];
    up.y = mat.Element[5];
    up.z = mat.Element[9];
    up.Normalize();
    up.x *= halfheight;
    up.y *= halfheight;
    up.z *= halfheight;

/*
   If you are using a quake style camera that doesn't roll
   then you can use the following commented code to make a conventional
   billboard remain vertical. If you are using a camera with 6DOF however,
   then this will not work.
*/

/*
    up.x = 0.0;
    up.z = 0.0;
    right.y = 0.0;
//*/

    float mat_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
    float mat_diffuse[] = { 0.8, 0.8, 0.8, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_ALPHA_TEST);
    glAlphaFunc(GL_GREATER, 0);
    glDisable(GL_LIGHTING);
    glColor4f(1.0, 1.0, 1.0, 1.0);
    glBindTexture(GL_TEXTURE_2D, texture[1].TexID);
    glPopMatrix();
        glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 0.0f); glVertex3f(light[currentLight].Position.x + (-right.x - up.x), light[currentLight].Position.y + (-right.y - up.y), light[currentLight].Position.z + (-right.z - up.z));
            glTexCoord2f(1.0f, 0.0f); glVertex3f(light[currentLight].Position.x + (right.x - up.x), light[currentLight].Position.y + (right.y - up.y), light[currentLight].Position.z + (right.z - up.z));
            glTexCoord2f(1.0f, 1.0f); glVertex3f(light[currentLight].Position.x + (right.x + up.x), light[currentLight].Position.y + (right.y + up.y), light[currentLight].Position.z + (right.z + up.z));
            glTexCoord2f(0.0f, 1.0f); glVertex3f(light[currentLight].Position.x + (up.x - right.x), light[currentLight].Position.y + (up.y - right.y), light[currentLight].Position.z + (up.z - right.z));
        glEnd();
    glPopMatrix();
    glEnable(GL_LIGHTING);
    glDisable(GL_ALPHA);
    glDisable(GL_BLEND);
}

//  A fire type billboard that rotates to face the camera but remains vertical
void DrawFire(TEXTURE* texture, LIGHT* light, CAMERA* camera)
{
    float halfwidth = 1.0;
    float halfheight = 2.0;
    VECTOR FirePosition(3.0, -2.0 + halfheight, -8.0);
//  Get the vector from the billboard position to the camera
    VECTOR A;
    A.x = (camera[currentCamera].Position.x - FirePosition.x);
    A.y = (camera[currentCamera].Position.y - FirePosition.y);
    A.z = (camera[currentCamera].Position.z - FirePosition.z);
    A.Normalize();
// Set a vector to the standard up vector
    VECTOR B;
    B.x = (0);
    B.y = (1);
    B.z = (0);
    B.Normalize();
// Take the cross product of these vectors to find a vector perpendicular to the camera
    VECTOR C = CrossVector(A, B);
    C.Normalize();

// Negate the perpendicular vector to make the billboard face the front
    VECTOR right;
    right.x = -C.x * halfwidth;
    right.y = -C.y * halfwidth;
    right.z = -C.z * halfwidth;

    VECTOR up;
    up.x = 0;
    up.y = 1 * halfheight;
    up.z = 0;

// Continue as normal
    float mat_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
    float mat_diffuse[] = { 0.8, 0.8, 0.8, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_ALPHA_TEST);
    glAlphaFunc(GL_GREATER, 0);
    glDisable(GL_LIGHTING);
    glColor4f(1.0, 1.0, 1.0, 1.0);
    glBindTexture(GL_TEXTURE_2D, texture[2].TexID);
    glPopMatrix();
        glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 0.0f); glVertex3f(FirePosition.x + (-right.x - up.x), FirePosition.y + (-right.y - up.y), FirePosition.z + (-right.z - up.z));
            glTexCoord2f(1.0f, 0.0f); glVertex3f(FirePosition.x + (right.x - up.x), FirePosition.y + (right.y - up.y), FirePosition.z + (right.z - up.z));
            glTexCoord2f(1.0f, 1.0f); glVertex3f(FirePosition.x + (right.x + up.x), FirePosition.y + (right.y + up.y), FirePosition.z + (right.z + up.z));
            glTexCoord2f(0.0f, 1.0f); glVertex3f(FirePosition.x + (up.x - right.x), FirePosition.y + (up.y - right.y), FirePosition.z + (up.z - right.z));
        glEnd();
    glPopMatrix();
    glEnable(GL_LIGHTING);
    glDisable(GL_ALPHA);
    glDisable(GL_BLEND);
}

void DrawBillboards(TEXTURE* texture, LIGHT* light, CAMERA* camera)
{
        DrawHalo(texture, light, camera);

/*
// Draw depth sorted billboards
    VECTOR FirePosition(3.0, -2.0, -8.0);

    VECTOR VectorToCone;
    VectorToCone.x = FirePosition.x - camera[currentCamera].Position.x;
    VectorToCone.y = FirePosition.y - camera[currentCamera].Position.y;
    VectorToCone.z = FirePosition.z - camera[currentCamera].Position.z;
    float FireDistance = VectorToCone.GetMagnitude();

    VECTOR VectorToLight;
    VectorToLight.x = light[currentLight].Position.x - camera[currentCamera].Position.x;
    VectorToLight.y = light[currentLight].Position.y - camera[currentCamera].Position.y;
    VectorToLight.z = light[currentLight].Position.z - camera[currentCamera].Position.z;
    float LightDistance = VectorToLight.GetMagnitude();

    if(LightDistance < FireDistance)
    {
        DrawFire();
        DrawHalo();
    }
    else
    {
        DrawHalo();
        DrawFire();
    }
//*/
}
