// PVS routines      by Alan Baylis 2002

#include "pvs.h"
#include "listnode.h"
#include "plane.h"
#include "portal.h"
#include "vertex.h"
#include "vector.h"
#include "camera.h"
#include "locmath.h"


extern int numleaves;
extern LinkedList<ListNode> LeafList;
extern LinkedList<ListNode> PartitionList;
extern ListNode* listnode;
extern CAMERA* camera;
extern int currentCamera;
extern PLANE frustum[6];

int CountVisibleLeaves()
{
    int counter = 0;
    ListNode* tempnode;
    for (int loop = 1; loop <= numleaves; loop++)
    {
        tempnode = LeafList.Get(loop);
        if (tempnode->node->visible)
        {
            counter++;
        }
    }
    return counter;
}

void FindVisibleLeaves(int Parent, PORTAL* CurrentPortal, ListNode* ParentNode, int portalnumber)
{
    int loop, numPlanes, leafnumber, Nminus1, counter, flag, result, parentid, planenumber;
    VECTOR edge1, edge2, pointOnPlane, planesNormal;
    ListNode* CurrentNode;
    PORTAL* front = new PORTAL;
    PORTAL* back = new PORTAL;
    PORTAL* inputPortal;

    inputPortal = ParentNode->node->portallist.Get(portalnumber);
    parentid = inputPortal->backleaf->nodeid;
    CurrentNode = LeafList.Get(parentid);
    CurrentNode->node->visible = true;

    // Create the planes from CurrentPortal
    pointOnPlane = camera[currentCamera].Position;

    numPlanes = CurrentPortal->numVertices;
    PLANE* Planes = new PLANE[numPlanes];
    for (loop = 0; loop < numPlanes; loop++)
    {
        if (loop == 0)
            Nminus1 = numPlanes - 1;
        else
            Nminus1 = loop - 1;
        // get the normal from edges
        edge1.x = CurrentPortal->Vertex[loop].x - camera[currentCamera].Position.x;
        edge1.y = CurrentPortal->Vertex[loop].y - camera[currentCamera].Position.y;
        edge1.z = CurrentPortal->Vertex[loop].z - camera[currentCamera].Position.z;
        edge2.x = CurrentPortal->Vertex[Nminus1].x - camera[currentCamera].Position.x;
        edge2.y = CurrentPortal->Vertex[Nminus1].y - camera[currentCamera].Position.y;
        edge2.z = CurrentPortal->Vertex[Nminus1].z - camera[currentCamera].Position.z;
        planesNormal = CrossVector(edge1, edge2);
        Planes[loop].nx = planesNormal.x;
        Planes[loop].ny = planesNormal.y;
        Planes[loop].nz = planesNormal.z;
    }

    // loop through the portals of this leafnode
    for (portalnumber = 1; portalnumber <= CurrentNode->node->numportals; portalnumber++)
    {
        counter = 0;
        CurrentPortal = CurrentNode->node->portallist.Get(portalnumber);
        flag = 1;
        // if the backleaf isn't the parent
        if (CurrentPortal->backleaf->nodeid != Parent)
        {
            // loop through the planes
            for (planenumber = 0; planenumber < numPlanes; planenumber++)
            {
                result = SplitPortal(CurrentPortal, Planes[planenumber], pointOnPlane, front, back);
                if (result == PortalWasSplit)
                {
                    delete[] back->Vertex;
                    if (flag == 0)   // if there has been a previous split polygon
                    {
                        delete[] CurrentPortal->Vertex;
                        delete CurrentPortal;
                    }
                    flag = 0;
                    CurrentPortal = front;
                    front = new PORTAL;
                }
                if (result != Back)
                    counter++;
            }
            // if the portal was split by or infront of all the planes
            if (counter == numPlanes)
            {
                FindVisibleLeaves(parentid, CurrentPortal, CurrentNode, portalnumber);
            }
            if (flag == 0)   // if there has been at least one split polygon
            {
                delete[] CurrentPortal->Vertex;
                delete CurrentPortal;
            }
        }
    }
    delete[] Planes;
    delete front;
    delete back;
}

void CalculatePVS(int currentleaf)
{
    int flag, result, counter, parentid, portalnumber, planenumber;
    VECTOR pointOnPlane;
    VECTOR ZUnit = camera[currentCamera].GetZUnit();    // camera's direction vector
    ListNode* CurrentNode;
    PORTAL* CurrentPortal;
    PORTAL* front = new PORTAL;
    PORTAL* back = new PORTAL;

    parentid = currentleaf;
    CurrentNode = LeafList.Get(currentleaf);
    CurrentNode->node->visible = true;
    // loop through the portals of the current leafnode
    for (portalnumber = 1; portalnumber <= CurrentNode->node->numportals; portalnumber++)
    {
        counter = 0;
        CurrentPortal = CurrentNode->node->portallist.Get(portalnumber);
        flag = 1;
        // loop through the frustum planes
        for (planenumber = 0; planenumber < 6; planenumber++)
        {
            // find a point on the far plane by projecting the camera's direction vector
            if (planenumber == 4)
            {
                pointOnPlane.x = camera[currentCamera].Position.x;
                pointOnPlane.x -= ZUnit.x * 500.0;
                pointOnPlane.y = camera[currentCamera].Position.y;
                pointOnPlane.y -= ZUnit.y * 500.0;
                pointOnPlane.z = camera[currentCamera].Position.z;
                pointOnPlane.z -= ZUnit.z * 500.0;
            }
            else if (planenumber == 5) // skip the near frustum plane test
                continue;
            else
                // the camera position is a point on the frustum planes
                pointOnPlane = camera[currentCamera].Position;

            result = SplitPortal(CurrentPortal, frustum[planenumber], pointOnPlane, front, back);
            if (result == PortalWasSplit)
            {
                delete[] back->Vertex;
                if (flag == 0)   // if there has been a previous split portal
                {
                    delete[] CurrentPortal->Vertex;
                    delete CurrentPortal;
                }
                flag = 0;
                CurrentPortal = front;
                front = new PORTAL;
            }
            if (result != Back)
                counter++;
        }
        // if the portal was split by or infront of the planes
        if (counter == 5)
        {
            FindVisibleLeaves(parentid, CurrentPortal, CurrentNode, portalnumber);
        }
        if (flag == 0)  // if there has been at least one split portal
        {
            delete[] CurrentPortal->Vertex;
            delete CurrentPortal;
        }
    }
    delete front;
    delete back;
}
